#
# 
# AutoMesh:
# Script for performing automatic mesh generation
# It is developed under contract to MIT for Auto Mesh Generation
# THIS SCRIPT IS STILL UNDER DEVELOPMENT
# THIS IS PRELIMINARY VERSION 1.01
#
# Written by Steve Karman, Pointwise, Inc.
#
# This Glyph script is not supported by Pointwise, Inc.
# It is provided freely for demonstration purposes only.  
# SEE THE WARRANTY DISCLAIMER AT THE BOTTOM OF THIS FILE.
#
#

# Load Glyph and TK
package require PWI_Glyph
pw::Script loadTK

if { ![info exists argv0] } {
  # Being run in Pointwise GUI
  set argv0 [info script]
  set argv {}
  set argc 0
}

#
# Initialize Pointwise
#
pw::Application reset
pw::Application clearModified

#
# Define working directory and load additional scripts
#
set scriptDir [file dirname [info script]]
source [file join $scriptDir "dbUtility.glf"]
source [file join $scriptDir "Utility.glf"]

#
# Initialize global variables defaults
#
source [file join $scriptDir "AutoMeshDefaults.glf"]

set CADFile ""

if { [pw::Application isInteractive] } {
  #
  # AUTOMESH GUI PANEL
  #
  wm title . "AutoMesh"
  grid [ttk::frame .c -padding "5 5 5 5"] -column 0 -row 0 -sticky nwes
  grid columnconfigure . 0 -weight 1; grid rowconfigure . 0 -weight 1
  grid [ttk::labelframe .c.lf -padding "5 5 5 5" -text "Automatic Meshing"]
  
  grid [ttk::label .c.lf.connl -text "Connector:"]                                          -row 1 -column 1 -sticky e
  grid [ttk::label .c.lf.cinitdiml -text "Initial Dimension"]                               -row 1 -column 2 -sticky e
  grid [ttk::entry .c.lf.cinitdime -width 5 -textvariable conInitDim]                       -row 1 -column 3 -sticky e
  grid [ttk::label .c.lf.cmindiml -text "Minimum Dimension"]                                -row 1 -column 4 -sticky e
  grid [ttk::entry .c.lf.cmindime -width 5 -textvariable conMinDim]                         -row 1 -column 5 -sticky e
  grid [ttk::label .c.lf.cmaxdiml -text "Maximum Dimension"]                                -row 1 -column 6 -sticky e
  grid [ttk::entry .c.lf.cmaxdime -width 5 -textvariable conMaxDim]                         -row 1 -column 7 -sticky e
  grid [ttk::label .c.lf.cturnl -text "Turning Angle"]                                      -row 2 -column 2 -sticky e
  grid [ttk::entry .c.lf.cturne -width 5 -textvariable conTurnAngle]                        -row 2 -column 3 -sticky e
  grid [ttk::label .c.lf.csplitl -text "Splitting Angle"]                                   -row 2 -column 4 -sticky e
  grid [ttk::entry .c.lf.csplite -width 5 -textvariable conSplitAngle]                      -row 2 -column 5 -sticky e
  grid [ttk::label .c.lf.cgrowthl -text "Proximity Growth Rate"]                            -row 2 -column 6 -sticky e
  grid [ttk::entry .c.lf.cgrowthe -width 5 -textvariable proxGrowthRate]                    -row 2 -column 7 -sticky e
  
  grid [ttk::label .c.lf.dtrexl -text "Domain T-Rex:"]                                      -row 3 -column 1 -sticky e
  grid [ttk::label .c.lf.dtfulll -text "Full Layers"]                                       -row 3 -column 2 -sticky e
  grid [ttk::entry .c.lf.dtfulle -width 5 -textvariable domainFullLayers]                   -row 3 -column 3 -sticky e
  grid [ttk::label .c.lf.dtmaxl -text "Maximum Layers"]                                     -row 3 -column 4 -sticky e
  grid [ttk::entry .c.lf.dtmaxe -width 5 -textvariable domainMaxLayers]                     -row 3 -column 5 -sticky e
  grid [ttk::label .c.lf.dtgrowthl -text "Growth Rate"]                                     -row 3 -column 6 -sticky e
  grid [ttk::entry .c.lf.dtgrowthe -width 5 -textvariable domainGrowthRate]                 -row 3 -column 7 -sticky e

  grid [ttk::label .c.lf.dmnl -text "Domain:"]                                              -row 4 -column 1 -sticky e
  grid [ttk::checkbutton .c.lf.tquad -text "Allow Tris & Quads" -variable IsoTriQuad]       -row 4 -column 2 -sticky e
  
  grid [ttk::label .c.lf.trexl -text "Volume T-Rex:"]                                       -row 5 -column 1 -sticky e
  grid [ttk::label .c.lf.tfulll -text "Full Layers"]                                        -row 5 -column 2 -sticky e
  grid [ttk::entry .c.lf.tfulle -width 5 -textvariable fullLayers]                          -row 5 -column 3 -sticky e
  grid [ttk::label .c.lf.tmaxl -text "Maximum Layers"]                                      -row 5 -column 4 -sticky e
  grid [ttk::entry .c.lf.tmaxe -width 5 -textvariable maxLayers]                            -row 5 -column 5 -sticky e
  grid [ttk::label .c.lf.tgrowthl -text "Growth Rate"]                                      -row 5 -column 6 -sticky e
  grid [ttk::entry .c.lf.tgrowthe -width 5 -textvariable growthRate]                        -row 5 -column 7 -sticky e
  grid [ttk::label .c.lf.tcolll -text "Collision Buffer"]                                   -row 5 -column 8 -sticky e
  grid [ttk::entry .c.lf.tcolle -width 5 -textvariable collisionBuffer]                     -row 5 -column 9 -sticky e
  
  grid [ttk::label .c.lf.voll -text "Volume:"]                                              -row 6 -column 1 -sticky e
  grid [ttk::label .c.lf.edgeratl -text "Edge Ratio"]                                       -row 6 -column 2 -sticky e
  grid [ttk::entry .c.lf.edgerate -width 5 -textvariable edgeMaxGrowthRate]                 -row 6 -column 3 -sticky e
  grid [ttk::label .c.lf.decayl -text "Boundary Decay"]                                     -row 6 -column 4 -sticky e
  grid [ttk::entry .c.lf.decaye -width 5 -textvariable boundaryDecay]                       -row 6 -column 5 -sticky e
  grid [ttk::checkbutton .c.lf.tprism -text "Allow Prisms & Pyramids" -variable TRexPrisms] -row 7 -column 2 -sticky e
  grid [ttk::checkbutton .c.lf.vinit -text "Initialize Volume" -variable volInitialize]     -row 7 -column 3 -sticky e
  
  grid [ttk::label .c.lf.cael -text "CAE Solver"]                                           -row 8 -column 1 -sticky e
  grid [ttk::entry .c.lf.caee -width 20 -textvariable CAESolver]                            -row 8 -column 2 -sticky e
  grid [ttk::button .c.lf.gob -text "Select CAD file" -command automesh]                    -row 8 -column 9 -sticky e
  foreach w [winfo children .c.lf] {grid configure $w -padx 10 -pady 10}
  focus .c.lf.cmindime
  ::tk::PlaceWindow . widget
  bind . <Return> {automesh}
  
} else {

  #
  # retrieve files from command line arguments
  #
  set numargs [llength $argv]
  puts "Number of command line arguments = $numargs"
  if { 0 == $numargs } {
    puts "Invalid command line input!"
    puts "Usage: PW_Application_Path [info script] GeomFile DefaultsFile"
    exit
  }

  for {set iarg 0} {$iarg < $argc} {incr iarg} {
    # first argument is the CAD file
    if { 0 == $iarg } {
      set CADFile [lindex $argv $iarg]
      puts "CADFile = $CADFile"
    }
    # second argument is the optional defaults file
    if { 1 == $iarg } {
      set dfile [lindex $argv $iarg]
      puts "Additional defaults file = $dfile" 
      source $dfile
    }
  }

}


#
# Main procedure
#
proc automesh { } {

# parameters from GUI or batch
global boundaryDecay collisionBuffer conInitDim conMaxDim conMinDim conTurnAngle conSplitAngle
global edgeMaxGrowthRate proxGrowthRate
global fullLayers maxLayers growthRate domainFullLayers domainMaxLayers domainGrowthRate
global IsoTriQuad TRexPrisms volInitialize
global CADFile CAESolver


#
# set PW defaults
#
set PWversion [pw::Application getVersion]
puts "Pointwise version $PWversion"

if { $conMinDim > $conInitDim } {
  set conInitDim $conMinDim
}
pw::Application setGridPreference Unstructured
pw::DomainUnstructured setDefault Algorithm AdvancingFront
pw::BlockUnstructured setDefault EdgeMaximumGrowthRate $edgeMaxGrowthRate
pw::Connector setDefault Dimension $conInitDim
pw::Connector setCalculateDimensionMaximum $conMaxDim
#pw::GridEntity setDefault SizeFieldDecay $boundaryDecay
pw::BlockUnstructured setDefault BoundaryDecay $boundaryDecay

#
# select NMB file
#
if { $CADFile == "" } {
  set CADFile [tk_getOpenFile]
}
puts "CADFile = $CADFile"

#
# load NMB file
#
set _TMP(mode_1) [pw::Application begin DatabaseImport]
  $_TMP(mode_1) initialize -strict -type Automatic $CADFile
  $_TMP(mode_1) read
  $_TMP(mode_1) convert
$_TMP(mode_1) end
unset _TMP(mode_1)
pw::Application markUndoLevel {Import Database}

pw::Display update

#
# apply quilting instructions from geometry
#
assembleTaggedQuilts

#
# gather quilts
#
set QuiltList {}

set dbEnts [pw::Database getAll]
puts "Dumping imported attributes for [llength $dbEnts] database entities"
foreach dbEnt $dbEnts {
  # uncomment next line to see details of attributes for each db entity
  #dumpAttrs $dbEnt
  switch -- [$dbEnt getDescription] {
  Quilt {
    puts "Quilt is linked to [$dbEnt getName]"
    lappend QuiltList [pw::DatabaseEntity getByName [$dbEnt getName]]
  }
  default {
  }}
}
puts "Quilt list has [llength $QuiltList] entries."

#
# set default connector and domain parameters
#
pw::Connector setDefault Dimension $conInitDim
if { 1 == $IsoTriQuad } {
  pw::DomainUnstructured setDefault IsoCellType TriangleQuad
} else {
  pw::DomainUnstructured setDefault IsoCellType Triangle
}

#
# create domains from quilts
# if unsuccessful for a given domain, then reset, increase the initial connector dimension and try again
#
set flag 0
while { 0 == $flag } {
  pw::Connector setDefault Dimension $conInitDim
  
  set flag 1
  set i 0
  foreach qlt $QuiltList {
  
    set domMode [pw::DomainUnstructured createOnDatabase -splitConnectors $conSplitAngle -parametricConnectors EndToEnd -merge 0 -reject unusedSurfs $qlt]
    if { [llength $unusedSurfs] > 0 } {
      puts "Quilt [expr $i+1]"
      puts "Unused surfaces exist."
      foreach surf $unusedSurfs {
        puts [$surf getName]
      }
      incr conInitDim 1
      puts "Increasing base connector dimension to $conInitDim"
      if {$conMinDim < $conInitDim} {
        set conMinDim $conInitDim
      }

      set flag 0

    }
    unset unusedSurfs
    unset domMode
  
    pw::Display update

    incr i 1

    if {$flag == 0} {
      break;
    }
  }
  if {$flag == 0} {
    # Reset system by force deleting all connectors
    set conList [pw::Grid getAll -type pw::Connector]
    set numcon [llength $conList]
    puts "Deleting all $numcon connectors"
    foreach con $conList {
      $con delete -force
    }
    puts "Deleted all $numcon connectors"
    unset conList
  }
}

pw::Display update

#
# compute tolerance for connector operations
#
set conList [pw::Grid getAll -type pw::Connector]
puts "Original connector list has [llength $conList] entries."

set minConLen 1.0e20
foreach con $conList {
  set len [$con getLength -parameter 1.0]
  if { $len < $minConLen } {
    set minConLen $len
  }
}
puts "Minimum connector length = $minConLen"
set tol [expr $minConLen / $conInitDim * 0.5]
puts "Tolerance = $tol"

#
# Perform merge operation to eliminate duplicate connectors
#
if {$conInitDim > 3} {
  if {0 < [connectorMergeUsingEndpoints $conList] } {
    unset conList
    set conList [pw::Grid getAll -type pw::Connector]
    puts "After endpoint based merge connector list has [llength $conList] entries."
  }
}

#
# Performing regular merge on connectors
#
set mergeMode [pw::Application begin Merge]
  $mergeMode mergeConnectors -visibleOnly -exclude None -tolerance $tol
$mergeMode end
unset mergeMode

unset conList
set conList [pw::Grid getAll -type pw::Connector]
puts "After regular merge, connector list has [llength $conList] entries."

#
# perform join operation and eliminate breakpoints
#
if {0 < [joinConnectors $conSplitAngle]} {
  unset conList
  set conList [pw::Grid getAll -type pw::Connector]
  puts "After join, connector list has [llength $conList] entries."
}

pw::Display update

#
# reduce connector dimension based on average spacing
#
reduceConnectorDimensionFromAvgSpacing $conMinDim $conMaxDim $conList

pw::Display update

#
# look for domains intersected by baffles
#
if { 0 < [processBaffleIntersections $tol] } {
  puts "Error processing baffles. Volume will not be initialized."
  set volInitialize 0
}

#
# Redimension connectors if turning angle is defined
#
if { 0.0 < $conTurnAngle } {

  increaseConnectorDimensionFromTurningAngle $conList $conMaxDim $conTurnAngle

}

unset conList

pw::Display update

set conList [pw::Grid getAll -type pw::Connector]

#
# Adjust connector dimension in close proximity to other connectors
#
connectorProximitySpacing $proxGrowthRate $conMinDim $conMaxDim $conList

pw::Display update

#
# Adjust connector dimension based on endpoint spacing
# and apply spacing values for tanh distribution
#
connectorDimensionFromEndSpacing $edgeMaxGrowthRate $conMinDim $conMaxDim $conList

pw::Display update

set domList [pw::Grid getAll -type pw::DomainUnstructured]
puts "Domain list has [llength $domList] entries."

if {[llength $domList] == 0} {
  exit -1
}
  
#
# search for domain attributes from geometry
#
loadDomainAttributes

pw::Display update

#
# set up domain T-Rex using end point spacing values
#
if { 0 < $domainMaxLayers } {
  setup2DTRexBoundaries $domList $domainFullLayers $domainMaxLayers $domainGrowthRate $boundaryDecay
}

foreach dom $domList {
  $dom setUnstructuredSolverAttribute BoundaryDecay $boundaryDecay
}

#
# do a refinement pass on all domains
#
puts "Performing refinement pass on all domains."
set refineMode [pw::Application begin UnstructuredSolver $domList]
  $refineMode run Refine
$refineMode end
unset refineMode

pw::Display update

#
# create single unstructured block
#
puts "Creating unstructured block from [llength $domList] domains."

#
# assembling domain list that excludes baffles
#
set bdomList {}
set rdomList {}
set numBaffles 0
foreach dom $domList {
  set baffle [domAttributeFromGeometry $dom "PW:Baffle"]
  if { "Baffle" == $baffle } {
    incr numBaffles 1
    puts "Baffle domain [$dom getName]"
    lappend bdomList $dom
  } else {
    lappend rdomList $dom
  }
}
if { 0 < $numBaffles } {
  puts "Number of baffles = $numBaffles"
}

set uBlk [pw::BlockUnstructured createFromDomains -reject unusedDoms $rdomList]

#
# Add baffles to block
#
if { 0 < $numBaffles } {
  set bmode [pw::Application begin Modify -notopology $uBlk]
  set bsub [subst {$uBlk}]
  foreach dom $bdomList {
    set ftmp [pw::FaceUnstructured create]
    $ftmp addDomain $dom
    $ftmp setBaffle true
    $bsub addFace $ftmp
    unset ftmp
  }
  $bmode end
  unset bmode
}
unset unusedDoms
unset rdomList
unset bdomList
pw::Application markUndoLevel {Assemble Blocks}

pw::Display update

set nblk [llength $uBlk]
if {1 > $nblk} {
  puts "Number of blocks = $nblk"
  exit -1
}

if { 1 == $nblk } {
  $uBlk setName "auto-blk"
  puts "Block = [$uBlk getName]"
}

#
# set up unstructured block
#
puts "Setting up unstructured block."
set solveMode [pw::Application begin UnstructuredSolver $uBlk]

  $uBlk setUnstructuredSolverAttribute TRexGrowthRate $growthRate
  $uBlk setUnstructuredSolverAttribute TRexFullLayers $fullLayers
  if {1 == $TRexPrisms} {
    $uBlk setUnstructuredSolverAttribute TRexCellType TetPyramidPrismHex
  } else {
    $uBlk setUnstructuredSolverAttribute TRexCellType TetPyramid
  }
  $uBlk setUnstructuredSolverAttribute TRexCollisionBuffer $collisionBuffer
  $uBlk setUnstructuredSolverAttribute BoundaryDecay $boundaryDecay

  # look for T-Rex setting in geometry
  set numLayers 0
  if {0 < $maxLayers} {
    if {1 == [setupTRexBoundaries $uBlk $domList]} {
      set numLayers $maxLayers
    }
  }
  
  $uBlk setUnstructuredSolverAttribute TRexMaximumLayers $numLayers

$solveMode end
unset solveMode
pw::Application markUndoLevel {Solve}

# initialize error count in case block initialization was not requested
# this prevents exporting of CAE data
set errorCount 1

#
# initialize unstructured block
#
if {1 == $volInitialize} {
  puts "Initializing unstructured block."
  set solveMode [pw::Application begin UnstructuredSolver $uBlk]
  
    $solveMode setStopWhenFullLayersNotMet true
    $solveMode run Initialize
  
  $solveMode end
  unset solveMode
  pw::Application markUndoLevel {Solve}

  set errorCount [$uBlk getInitializationErrorCount]
}

if { "" != $CAESolver } {
  set flag 0
  set solverNames [pw::Application getCAESolverNames]
  foreach solver $solverNames {
    if { $solver == $CAESolver } {
      pw::Application setCAESolver $CAESolver
      puts "CAE Solver set to $CAESolver"
      set flag 1
      break;
    }
  }
  if { 0 == $flag } {
    set CAESolver ""
  }
  unset flag
}

puts "AutoMesh finished!"

set fname "[file rootname $CADFile].AutoMesh.pw"
puts "Pointwise save filename = $fname"
pw::Application save $fname

#
#export solver data
#

if { 0 < $errorCount } {
  puts "Block not initialized."
}

if { "" != $CAESolver && 0 == $errorCount} {

  set basename "[file rootname $CADFile].AutoMesh"

  if { "CGNS" == $CAESolver } {
    set fname "[file rootname $CADFile].AutoMesh.cgns"
    puts "Exporting in CGNS format to filename = $fname"
    pw::Application export -precision Double $uBlk $fname
  }

  if { "UGRID" == $CAESolver } {
    puts "Exporting in UGRID format to file basename = $basename"
    pw::Application export -precision Double -format Binary $uBlk $basename
  }
}

exit

}

if { $CADFile != "" } {
  automesh
}

# END SCRIPT

# DISCLAIMER:
# TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, POINTWISE DISCLAIMS
# ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED
# TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE, WITH REGARD TO THIS SCRIPT.  TO THE MAXIMUM EXTENT PERMITTED 
# BY APPLICABLE LAW, IN NO EVENT SHALL POINTWISE BE LIABLE TO ANY PARTY 
# FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES 
# WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF 
# BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS) ARISING OUT OF THE 
# USE OF OR INABILITY TO USE THIS SCRIPT EVEN IF POINTWISE HAS BEEN 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND REGARDLESS OF THE 
# FAULT OR NEGLIGENCE OF POINTWISE.
#
