#
# General utilities for AutoMesh.glf
#
# THIS SCRIPT IS STILL UNDER DEVELOPMENT
# THIS IS A PRELIMINARY VERSION
#

#
# Written by Steve Karman, Pointwise, Inc.
#
# This Glyph script is not supported by Pointwise, Inc.
# It is provided freely for demonstration purposes only.  
# SEE THE WARRANTY DISCLAIMER AT THE BOTTOM OF THIS FILE.
#

proc lremove {var value} {
  #
  # delete a given element from a list
  #
  set idx [lsearch -exact $var $value]
  set var [lreplace $var $idx $idx]

  return $var
}

proc decaySpacing { sp distance decay spaceMax} {
  #
  # Decay spacing value over distance
  #

  set value $sp

  set mag [expr $distance - $value * 2 * $decay]
  while { 0.0 < $mag && $value < $spaceMax } {
    set mag [expr $mag - $value * 2 * $decay]
    set value [expr $value * $decay]
  }

  return $value
}

proc domAttributeFromGeometry { dom keyword } {
  #
  # Retrieve keyword and value from domain geometry
  #

  set value ""

  set dbEnts [$dom getDatabaseEntities]

  foreach dbEnt $dbEnts {
    set nVal [attributeValue $dbEnt "TrimmedSurface" $keyword ]
    if {"" != $nVal } {
      set value $nVal
      break
    }
  }

  return $value
}

proc loadDomainAttributes { } {
  #
  # Retrieve domain attributes from geometry
  #

  set domList [pw::Grid getAll -type pw::DomainUnstructured]
  
  set i 0
  foreach dom $domList {

    # look for boundary names
    set name [domAttributeFromGeometry $dom "PW:Name"]
    if { "" != $name } {
      puts "Domain [expr $i+1] of [llength $domList] boundary name = $name."
      set bnames [pw::BoundaryCondition getNames]
      set sflag 0
      foreach bname $bnames {
        if {$bname == $name} {
          puts "$name already in boundary name list."
          set sflag 1
          set bc [pw::BoundaryCondition getByName $name]
          $bc apply $dom
        }
      }
      if {$sflag == 0} {
        puts "Creating boundary name $name"
        set bc [pw::BoundaryCondition create]
        $bc setName $name
        $bc apply $dom
      }
    }
    unset name

    # look for domain cell types
    set type [domAttributeFromGeometry $dom "PW:SurfaceCellType"]
    if { "Triangle" == $type || "TriangleQuad" == $type } {
      puts "Domain [expr $i+1] of [llength $domList], cell type = $type."
      set solveMode [pw::Application begin UnstructuredSolver [list $dom]]
        set tmpEnts [pw::Collection create]
        $tmpEnts set [list $dom] 
        $dom setUnstructuredSolverAttribute IsoCellType $type
        $tmpEnts delete
        unset tmpEnts
        $solveMode run Refine
      $solveMode end
      unset solveMode
    }
    unset type

    # look for mesh algorithm
    set algorithm [domAttributeFromGeometry $dom "PW:SurfaceAlgorithm"]
    if { "Delaunay" == $algorithm || "AdvancingFront" == $algorithm || "AdvancingFrontOrtho" == $algorithm} {
      puts "Domain [expr $i+1] of [llength $domList], mesh algorithm = $algorithm."
      set solveMode [pw::Application begin UnstructuredSolver [list $dom]]
        set tmpEnts [pw::Collection create]
        $tmpEnts set [list $dom] 
        $dom setUnstructuredSolverAttribute Algorithm $algorithm
        $tmpEnts delete
        unset tmpEnts
        $solveMode run Refine
      $solveMode end
      unset solveMode
    }
    unset algorithm

    incr i 1
  }
  
  unset domList

}

proc connectorProximitySpacing { decay conMinDim conMaxDim conList } {
  #
  # connectors tested for close proximity to other connectors.
  #  for each connector, compute distance-decayed spacing from other connectors not attached to current connector
  #   if spacing is too large increase connector dimension accordingly
  #

  puts "Adjusting connector dimensions based on proximity to other connectors."

  # store initial spacing for each connector
  set conSpacing {}
  foreach con $conList {
    set dim [$con getDimension]
    if { $dim < [expr max(2,$conMinDim)] } {
      set newdim $conMinDim
      $con setDimension $newdim
    }
    set sp [$con getAverageSpacing]
    lappend conSpacing [list $con $sp]
  }
  
  set numcons [llength $conList]
  
  # check spacing of current connector with all shorter connectors
  for { set i 0 } { $i < $numcons } { incr i } {
    set con [lindex [lindex $conSpacing $i] 0]
    set sp [lindex [lindex $conSpacing $i] 1]
  
    set adjList {}
    set n0 [$con getNode Begin]
    set conns [$n0 getConnectors]
    foreach acon $conns {
      set j [lsearch $adjList $acon]
      if { -1 == $j } {
        lappend adjList $acon
      }
      unset j
    }
    set n1 [$con getNode End]
    set conns [$n1 getConnectors]
    foreach acon $conns {
      set j [lsearch $adjList $acon]
      if { -1 == $j } {
        lappend adjList $acon
      }
      unset j
    }
  
    # determine minimum decayed spacing from shorter connectors
    set dim [$con getDimension]
    set minsp [expr 1.1*$sp]
    for { set j 0} { $j < $numcons } { incr j } {
  
      if { $j != $i } {
        set tcon [lindex [lindex $conSpacing $j] 0]
        set tsp [lindex [lindex $conSpacing $j] 1]
        set k [lsearch $adjList $tcon]
        if { -1 == $k && $tsp < $minsp } {
          set minDS 1.0e20
          for { set n 2 } { $n < $dim } { incr n } {
            set pt [$con getXYZ -grid $n]
            set dp [$tcon closestPoint -distance ds $pt]
            if { $ds < $minDS } {
              set minDS $ds
            }
          }
          # decay spacing over distance
          set newsp [decaySpacing $tsp $minDS $decay $sp]
          if { $newsp < $minsp } {
            set minsp $newsp
          }
        }
      }
    }
    # if decayed spacing is smaller then re-dimension connector
    if { $minsp < $sp } {
      set newdim [$con setDimensionFromSpacing $minsp]
      set newsp [$con getAverageSpacing]
      if {$newdim > $conMaxDim} {
        set newdim [$con setDimension $conMaxDim]
        set newsp [$con getAverageSpacing]
      }
      puts "Proximity test for connector $i/$numcons, dimension changed from $dim to $newdim, sp changed from $sp to $newsp"
    }
  
    unset adjList
  
  }

  unset conSpacing

}

proc WallSpacingFromGeometry { dom } {
  #
  # Retrieve wall normal spacing from geometry
  #

  set sp 0.0

  set dbEnts [$dom getDatabaseEntities]

  foreach dbEnt $dbEnts {
    set sVal [attributeValue $dbEnt "TrimmedSurface" "PW:WallSpacing"]
    if {"" != $sVal } {
      set sp $sVal
      break
    }
  }

  return $sp

}

proc connectorMergeUsingEndpoints { conList } {
  #
  # for each connector, check whether other connectors have the same two endpoints and dimension
  # and if the middle points are the same within a tolerance then merge the connectors
  #

  puts "Performing merge on connectors with equal dimensions and common end points."

  set replaceList {}

  foreach con $conList {
    set condim [$con getDimension]
    set n0 [$con getNode Begin]
    set n1 [$con getNode End]
    set len [$con getLength -parameter 1.0]
    set tol [expr $len / ($condim-1.0) * 0.1]

    set n0cons [$n0 getConnectors]
    set n1cons [$n1 getConnectors]

    # select connector at n0 not equal to current with same dimension
    foreach con0 $n0cons {
      if { ($con0 != $con) && ($condim == [$con0 getDimension]) } {
        # search for same connector at n1
        foreach con1 $n1cons {
          if { $con0 == $con1 } {
   
            # check forward match
            set match_f 0
            if { $n0 == [$con0 getNode Begin] && $n1 == [$con0 getNode End] } {
              set match_f 1
              for { set n 2 } { $n < $condim } { incr n } {
                set pt0 [$con getXYZ -grid $n]
                set pt1 [$con1 getXYZ -grid $n]
                set dx [expr [lindex $pt1 0] - [lindex $pt0 0]]
                set dy [expr [lindex $pt1 1] - [lindex $pt0 1]]
                set dz [expr [lindex $pt1 2] - [lindex $pt0 2]]
                set ds [expr ($dx**2 + $dy**2 + $dz**2)]
                set ds [expr sqrt($ds) ]
                if { $ds > $tol } {
                  set match_f 0
                  break
                }
              }
            } 

            # check backward match
            set match_b 0
            if { $n1 == [$con0 getNode Begin] && $n0 == [$con0 getNode End] } {
              set match_b 1
              for { set n 2 } { $n < $condim } { incr n } {
                set m [expr $condim - $n + 1]
                set pt0 [$con getXYZ -grid $n]
                set pt1 [$con1 getXYZ -grid $m]
                set dx [expr [lindex $pt1 0] - [lindex $pt0 0]]
                set dy [expr [lindex $pt1 1] - [lindex $pt0 1]]
                set dz [expr [lindex $pt1 2] - [lindex $pt0 2]]
                set ds [expr ($dx**2 + $dy**2 + $dz**2)]
                set ds [expr sqrt($ds) ]
                if { $ds > $tol } {
                  set match_b 0
                  break
                }
              }

            }

            # if either direction is a match then add to merge list
            if { 1 == $match_f || 1 == $match_b } {
              # has it been save already?
              set flag 0
              foreach pair $replaceList {
                set p0 [lindex $pair 0]
                set p1 [lindex $pair 1]

                if { $p0 == $con && $p1 == $con1 } {
                  set flag 1
                } elseif { $p1 == $con && $p0 == $con1 } {
                  set flag 1
                }
                if { $flag == 1 } {
                  break
                }
              }

              # add to list
              if { $flag == 0 } {
                lappend replaceList [list $con $con1]
              }
            }
          }
        }
      }
    }
  }

  set nmerge [llength $replaceList]
  if { 0 < $nmerge } {
    puts "Number of merge pairs identified = $nmerge"

    set mergeMode [pw::Application begin Merge]
      foreach pair $replaceList {
        set con0 [lindex $pair 0]
        set con1 [lindex $pair 1]
        set cname0 [$con0 getName]
        set cname1 [$con1 getName]
        puts "Replacing $cname1 with $cname0"
        $mergeMode replace $con0 $con1
      }
    $mergeMode end
    unset mergeMode
  }

  unset replaceList

  return $nmerge

}

proc increaseConnectorDimensionFromTurningAngle { conList conMaxDim conTurnAngle } {
  #
  # Redimension connectors if turning angle is defined
  #
  if { 0.0 < $conTurnAngle } {
    puts "Adjusting connector dimensions based on turning angles."
    set conMode [pw::Application begin Modify $conList]
    set i 0
    foreach con $conList {
      set sp [$con getAverageSpacing]
      incr i 1
      set dim [$con getDimension]
      set newdim [$con increaseDimension -surface $conTurnAngle 0.0]
      if { $newdim > $conMaxDim } {
        set newdim [$con setDimension $conMaxDim]
        $con setDimension $newdim
      }
      $con replaceDistribution 1 [pw::DistributionTanh create]
      set newsp [$con getAverageSpacing]
      if { $newdim > $dim } {
        puts "Turning angle test for connector $i/[llength $conList]. Dim changed from $dim to $newdim. Spacing changed from $sp to $newsp."
      }
    }
    $conMode end
    unset conMode
  }
}

proc reduceConnectorDimensionFromAvgSpacing { conMinDim conMaxDim conList } {
  #
  # This procedure will coarsen the mesh to allow smaller length connectors to be 
  # dimensioned to the minimum requested.
  #

  # Enforce minimum connector dimension
  foreach con $conList {
    set dim [$con getDimension]
    if { $dim < [expr max(2,$conMinDim)] } {
      $con setDistribution 1 [pw::DistributionTanh create]
      set newdim $conMinDim
      $con setDimension $newdim
    }
  }

  # create unique endpoint node list and minimum spacing value
  set nodeList {}
  set nodeSpacing {}
  
  # create maximum spacing values at nodes
  foreach con $conList {
    set sp [$con getAverageSpacing]
    set nn [[$con getNode Begin] getName]
    set i [lsearch $nodeList $nn]
    if { -1 == $i} {
      lappend nodeList $nn
      lappend nodeSpacing $sp
    } else {
      if { [lindex $nodeSpacing $i] < $sp} {
        set nodeSpacing [lreplace $nodeSpacing $i $i $sp]
      }
    }
    set nn [[$con getNode End] getName]
    set i [lsearch $nodeList $nn]
    if { -1 == $i } {
      lappend nodeList $nn
      lappend nodeSpacing $sp
    } else {
      if { [lindex $nodeSpacing $i] < $sp} {
        set nodeSpacing [lreplace $nodeSpacing $i $i $sp]
      }
    }
  }

  # create tolerance
  set tol 1.0e20
  set i 0
  foreach sp $nodeSpacing {
    incr i 1
    if { $sp < $tol } {
      set tol $sp
    }
  }
  set tol [expr $tol * 0.1]
  
  # Adjust connector dimensions
  puts "Adjusting connector dimensions based on average spacings."
  set conMode [pw::Application begin Modify $conList]
  set i 0
  foreach con $conList {
    incr i 1
    set sp [$con getAverageSpacing]
    set n0 [[$con getNode Begin] getName]
    set n1 [[$con getNode End] getName]
    set s0 [lindex $nodeSpacing [lsearch $nodeList $n0]]
    set s1 [lindex $nodeSpacing [lsearch $nodeList $n1]]
  
    if { [expr (($s0 - $sp)/$s0) ] > 0.01 || [expr (($s1 - $sp)/$s1) ] > 0.01 } {

      set olddim [$con getDimension]

      # compute new spacing based on length and max endpoint spacing
      set len [$con getLength -parameter 1.0]
      set newdim [expr int($len/max($s0,$s1))]

      if { $newdim < $conMinDim } {
        set newdim $conMinDim
      }
      if { $newdim != $olddim } {
        $con setDimension $newdim

        puts "Changing dimension for connector $i/[llength $conList] from $olddim to $newdim"
      }
      
    }
  }
  $conMode end

  unset nodeList
  unset nodeSpacing

}

proc connectorDimensionFromEndSpacing { edgeMaxGrowthRate conMinDim conMaxDim conList } {
  #
  # Adjust connector dimension up or down based on end point spacing values
  #

  # Enforce minimum connector dimension
  foreach con $conList {
    set dim [$con getDimension]
    if { $dim < [expr max(2,$conMinDim)] } {
      $con setDistribution 1 [pw::DistributionTanh create]
      set newdim $conMinDim
      $con setDimension $newdim
    }
  }

  # create unique endpoint node list and minimum spacing value
  set nodeList {}
  set nodeSpacing {}
  
  foreach con $conList {
    set sp [$con getAverageSpacing]
    set ni [$con getNode Begin]
    set nn [[$con getNode Begin] getName]
    set i [lsearch $nodeList $nn]
    if { -1 == $i} {
      lappend nodeList $nn
      lappend nodeSpacing $sp
    } else {
      if { [lindex $nodeSpacing $i] > $sp} {
        set nodeSpacing [lreplace $nodeSpacing $i $i $sp]
      }
    }
    set ni [$con getNode End]
    set nn [[$con getNode End] getName]
    set i [lsearch $nodeList $nn]
    if { -1 == $i } {
      lappend nodeList $nn
      lappend nodeSpacing $sp
    } else {
      if { [lindex $nodeSpacing $i] > $sp} {
        set nodeSpacing [lreplace $nodeSpacing $i $i $sp]
      }
    }
  }

  # create tolerance
  set tol 1.0e20
  set i 0
  foreach sp $nodeSpacing {
    incr i 1
    if { $sp < $tol } {
      set tol $sp
    }
  }
  set tol [expr $tol * 0.1]
  
  # Adjust connector dimensions and spacings
  puts "Adjusting connector dimensions based on end spacings."
  set conMode [pw::Application begin Modify $conList]
  set i 0
  foreach con $conList {
    incr i 1
    set sp [$con getAverageSpacing]
    set n0 [[$con getNode Begin] getName]
    set n1 [[$con getNode End] getName]
    set s0 [lindex $nodeSpacing [lsearch $nodeList $n0]]
    set s1 [lindex $nodeSpacing [lsearch $nodeList $n1]]
  
    # adjust dimension if average is different from end point spacing
    #   and apply tanh distribution
    if { [expr abs(($sp - $s0)/$sp) ] > 0.01 || [expr abs(($sp - $s1)/$sp) ] > 0.01 } {

      set dim [$con getDimension]

      # Set connector distribution type
      $con setDistribution 1 [pw::DistributionTanh create]

      set len [$con getLength -parameter 1.0]
      if { [expr $len/$s0] < 2.0 || [expr $len/$s1] < 2.0 } {
        set newdim [expr $dim + 1]
        $con setDimension $newdim
        set sp [$con getAverageSpacing]
      } else {
        set conDist [$con getDistribution 1]
        $conDist setBeginSpacing [expr min($sp,$s0)]
        $conDist setEndSpacing [expr min($sp,$s1)]
      }

      # Increase/decrease dimension until segment max growth rate is met
      set flag 0
      set dflag 0
      set iflag 0
      set olddim [$con getDimension]
      while { 0 == $flag } {
  
        set dim [$con getDimension]
        set maxRatio 1.0
        set minRatio 1.0
        set minds 1.0e20
        set flag 1
        if { 3 < $dim && $dim < $conMaxDim } {
          for { set n 1 } { $n < $dim-2 } { incr n } {
            set pt0 [$con getXYZ -grid $n]
            set pt1 [$con getXYZ -grid [expr $n + 1]]
            set pt2 [$con getXYZ -grid [expr $n + 2]]
            set dx [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set ds1 [expr ($dx**2 + $dy**2 + $dz**2)]
            set ds1 [expr sqrt($ds1) ]
            if { $n > 1 && $ds1 < $minds } {
              set minds $ds1
            }
            set dx [expr [lindex $pt2 0] - [lindex $pt1 0]]
            set dy [expr [lindex $pt2 1] - [lindex $pt1 1]]
            set dz [expr [lindex $pt2 2] - [lindex $pt1 2]]
            set ds2 [expr ($dx**2 + $dy**2 + $dz**2)]
            set ds2 [expr sqrt($ds2) ]
            if { $ds1 > $tol && $ds2 > $tol } {
              set ratio [expr {max($ds1,$ds2)} / {min($ds1,$ds2)}]
              if { $ratio > $maxRatio } {
                set maxRatio $ratio
              }
  
              set ratio [expr { $ds1 / min($s0,$s1) }]
              if { $ratio < $minRatio } {
                set minRatio $ratio
              }
              set ratio [expr { $ds2 / min($s0,$s1) }]
              if { $ratio < $minRatio } {
                set minRatio $ratio
              }
            }
          }
        }
  
        if { $dim < $conMinDim } {
          set newdim $conMinDim
          $con setDimension $newdim
          set iflag 1
          set flag 0
        } elseif { 0 == $iflag && $minRatio < [expr 1.0/$edgeMaxGrowthRate] && $dim > $conMinDim} {
          set newdim [expr $dim - 1]
          $con setDimension $newdim
          set dflag 1
          set flag 0
        } elseif { 0 == $dflag && $maxRatio > $edgeMaxGrowthRate && $dim < $conMaxDim} {
          # if internal minimum spacing is larger than end spacing increase dimension
          if { $minds > [expr min($s0,$s1)] } {
            set newdim [expr $dim + 1]
            $con setDimension $newdim
            set iflag 1
            set flag 0
          }
        }

      }
      if {$dim != $olddim} {
        puts "Changing dimension for connector $i/[llength $conList] from $olddim to $dim"
      }
      
    }
  }
  $conMode end

  unset nodeList
  unset nodeSpacing

}

proc setup2DTRexBoundaries { domList fullLayers maxLayers growthRate boundaryDecay } {
  #
  # set T-Rex domain boundary conditions
  #  based on connector endpoint spacing compared to average
  #

  puts "Setting up T-Rex domain boundary conditions."

  foreach dom $domList {

    set numedges [$dom getEdgeCount]

    set tflag 0
    for { set e 1 } { $e <= $numedges } { incr e } {
      set ed [$dom getEdge $e]
      set numcons [$ed getConnectorCount]
      for {set i 1 } { $i <= $numcons } { incr i } {
        set con [$ed getConnector $i]

        set dim [$con getDimension]
        set avgsp [$con getAverageSpacing]

        set pt0 [$con getXYZ -grid 1]
        set pt1 [$con getXYZ -grid 2]
        set dx [expr [lindex $pt1 0] - [lindex $pt0 0]]
        set dy [expr [lindex $pt1 1] - [lindex $pt0 1]]
        set dz [expr [lindex $pt1 2] - [lindex $pt0 2]]
        set s0 [expr sqrt($dx**2 + $dy**2 + $dz**2)]

        set pt0 [$con getXYZ -grid [expr {$dim - 1}]]
        set pt1 [$con getXYZ -grid $dim]
        set dx [expr [lindex $pt1 0] - [lindex $pt0 0]]
        set dy [expr [lindex $pt1 1] - [lindex $pt0 1]]
        set dz [expr [lindex $pt1 2] - [lindex $pt0 2]]
        set s1 [expr sqrt($dx**2 + $dy**2 + $dz**2)]

        if { [expr abs(($avgsp - $s0)/$avgsp) ] > 0.5 || [expr abs(($avgsp - $s1)/$avgsp) ] > 0.5 } {
          set sp [expr 0.5 * ($s0 + $s1)]

          set name "[$con getName]"
          set bnames [pw::TRexCondition getNames]

          set sflag 0
          foreach bname $bnames {
            if {$bname == $name} {
              puts "$name already in T-Rex boundary name list."
              set sflag 1
              set bc [pw::TRexCondition getByName $name]
                $bc setSpacing $sp
                $bc apply [list [list $dom $con]]
              break
            }
          }
          if {$sflag == 0} {
            # create the T-Rex BC
            puts "Creating T-Rex boundary name $name"
            set bc [pw::TRexCondition create]
              $bc apply [list [list $dom $con]]
              $bc setName $name
              $bc setConditionType {Wall}
              $bc setSpacing $sp
            unset bc
          }
          set tflag 1
        }
      }
    }
    if { 1 == $tflag } {
      $dom setUnstructuredSolverAttribute TRexFullLayers $fullLayers
      $dom setUnstructuredSolverAttribute TRexMaximumLayers $maxLayers
      $dom setUnstructuredSolverAttribute TRexGrowthRate $growthRate
      $dom setUnstructuredSolverAttribute BoundaryDecay $boundaryDecay
    }
  }

}

proc setupTRexBoundaries { uBlk domList } {
  #
  # set T-Rex volume boundary conditions
  #

  puts "Retrieving T-Rex volume boundary conditions from geometry."

  set lflag 0

  set tdomList {}
  foreach dom $domList {
    set sp [WallSpacingFromGeometry $dom]
    if { 0.0 < $sp } {
      set name [domAttributeFromGeometry $dom "PW:Name"]
      puts "Domain $name has spacing value = $sp"
      lappend tdomList $dom

      if { "" == $name } {
        puts "Creating T-Rex boundary name [$dom getName]"
        set bc [pw::TRexCondition create]
          $bc apply [list [list $uBlk $dom]]
          $bc setName [$dom getName]
          $bc setConditionType {Wall}
          $bc setSpacing $sp
        set lflag 1
      } else {
        set bnames [pw::TRexCondition getNames]

        set sflag 0
        foreach bname $bnames {
          if {$bname == $name} {
            puts "$name already in T-Rex boundary name list."
            set sflag 1
            set bc [pw::TRexCondition getByName $name]
              $bc apply [list [list $uBlk $dom]]
            set lflag 1
            break
          }
        }
        if {$sflag == 0} {
          puts "Creating T-Rex boundary name $name"
          set bc [pw::TRexCondition create]
            $bc apply [list [list $uBlk $dom]]
            $bc setName $name
            $bc setConditionType {Wall}
            $bc setSpacing $sp
          set lflag 1
        }
        unset bnames
      }
    }
  }

  # look for adjacent domains that need to be "match" boundaries
  if { 1 == $lflag  && [llength tdomList] > 0 } {

    puts "Looking for adjacent boundaries to set as match."

    set pflag 0

    foreach dom $tdomList {
      set adjDomList [pw::Domain getAdjacentDomains -all $dom]

      foreach adj $adjDomList {
        set i [lsearch $tdomList $adj]
        if { -1 == $i } {
          # Look for existence of Match bc type
          set bnames [pw::TRexCondition getNames]
          set mflag 0
          set name "Match"
          foreach bname $bnames {
            if {$bname == $name} {
              set mflag 1
            }
          }
          unset bnames
          if { 0 == $mflag } {
            puts "No match type BC present."
            set bc [pw::TRexCondition create]
              $bc setName $name
              $bc setConditionType {Match}
            unset bc
          }
          unset mflag
          unset name

          set bc [pw::TRexCondition getByName {Match}]
          $bc apply [list [list $uBlk $adj Opposite]]
          unset bc
          set pflag 1
        }
      }
      unset adjDomList
    }
    if { 1 == $pflag } {
      puts "Setting Push Attribute on Block."
      $uBlk setUnstructuredSolverAttribute TRexPushAttributes True
    }
    unset pflag
  }

  unset tdomList

  return $lflag
}

proc joinConnectors { conSplitAngle } {
  #
  # perform join on connectors where two connectors attached to any node
  #

  puts "Performing join operation on connectors."

  # Iteratively look for candidate connector pairs sharing a node with no other connectors
  # If a turning angle is specified and the two connectors qualify then add them to join list.
  # If a turning angle is not specified add to the list.
  # Repeat until no more candidates found

  set radSplitAngle 0.0
  puts "Turning angle threshold in degrees = $conSplitAngle"
  if { 1.0e-10 < $conSplitAngle } {
    set pi [expr 4.0*atan(1.0)]
    set radSplitAngle [expr $conSplitAngle * $pi / 180.0 ]
    puts "Turning angle threshold in radians = $radSplitAngle"
  }

  set rflag 0
  set Joined 1
  set pass 0
  while { 0 < $Joined } {

    incr pass 1
    puts "Join pass $pass."

    set nodeList {}

    set conList [pw::Grid getAll -type pw::Connector]

    foreach con $conList {
      set n [$con getNode Begin]
      set i [lsearch $nodeList $n]
      if { -1 == $i} {
        lappend nodeList $n
      }
      set n [$con getNode End]
      set i [lsearch $nodeList $n]
      if { -1 == $i} {
        lappend nodeList $n
      }
    }

    unset conList

    # create list of candidate connector pairs
    set pairList {}

    foreach n $nodeList {

      set cons [$n getConnectors]

      if { [llength $cons] == 2 } {
        set c0 [lindex $cons 0]
        set c1 [lindex $cons 1]
        set angle -1.0
        if { 0.0 < $radSplitAngle } {
          # compute the turning angle to determine
          # whether these two connectors need to remain separate
          set c0n0 [$c0 getNode Begin]
          set c0n1 [$c0 getNode End]
          set c1n0 [$c1 getNode Begin]
          set c1n1 [$c1 getNode End]
          set dim0 [$c0 getDimension]
          set dim1 [$c1 getDimension]
          if { $c0n0 == $c1n0 } {
            set pt0 [$c0 getXYZ -grid 1]
            set pt1 [$c0 getXYZ -grid 2]
            set dx0 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy0 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz0 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v0 [pwu::Vector3 set $dx0 $dy0 $dz0]
            set pt0 [$c1 getXYZ -grid 2]
            set pt1 [$c1 getXYZ -grid 1]
            set dx1 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy1 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz1 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v1 [pwu::Vector3 set $dx1 $dy1 $dz1]
          }
          if { $c0n0 == $c1n1 } {
            set pt0 [$c0 getXYZ -grid 1]
            set pt1 [$c0 getXYZ -grid 2]
            set dx0 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy0 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz0 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v0 [pwu::Vector3 set $dx0 $dy0 $dz0]
            set pt0 [$c1 getXYZ -grid [expr { $dim1 - 1} ]]
            set pt1 [$c1 getXYZ -grid $dim1]
            set dx1 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy1 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz1 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v1 [pwu::Vector3 set $dx1 $dy1 $dz1]
          }
          if { $c0n1 == $c1n0 } {
            set pt0 [$c0 getXYZ -grid $dim0]
            set pt1 [$c0 getXYZ -grid [expr {$dim0 - 1} ]]
            set dx0 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy0 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz0 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v0 [pwu::Vector3 set $dx0 $dy0 $dz0]
            set pt0 [$c1 getXYZ -grid 2]
            set pt1 [$c1 getXYZ -grid 1]
            set dx1 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy1 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz1 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v1 [pwu::Vector3 set $dx1 $dy1 $dz1]
          }
          if { $c0n1 == $c1n1 } {
            set pt0 [$c0 getXYZ -grid $dim0]
            set pt1 [$c0 getXYZ -grid [expr {$dim0 - 1} ]]
            set dx0 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy0 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz0 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v0 [pwu::Vector3 set $dx0 $dy0 $dz0]
            set pt0 [$c1 getXYZ -grid [expr { $dim1 - 1} ]]
            set pt1 [$c1 getXYZ -grid $dim1]
            set dx1 [expr [lindex $pt1 0] - [lindex $pt0 0]]
            set dy1 [expr [lindex $pt1 1] - [lindex $pt0 1]]
            set dz1 [expr [lindex $pt1 2] - [lindex $pt0 2]]
            set v1 [pwu::Vector3 set $dx1 $dy1 $dz1]
          }

          set vn0 [pwu::Vector3 normalize $v0]
          set vn1 [pwu::Vector3 normalize $v1]
          set dot [pwu::Vector3 dot $vn0 $vn1]
          set adot [expr $dot]
          set angle [expr acos($adot)]
          puts "Angle between [$c0 getName] and [$c1 getName] = $angle radians"
        }
        if { 1.0e-10 > $conSplitAngle || $angle < $radSplitAngle } {
          set found 0
          foreach pair $pairList {
            set p0 [lindex $pair 0]
            set p1 [lindex $pair 1]
            if { $p0 == $c0 || $p0 == $c1 || $p1 == $c0 || $p1 == $c1 } {
              set found 1
              break
            }
          }
          if { 0 == $found } {
            lappend pairList [list $c0 $c1]
          }
        }
      }
      unset cons
    }

    set Joined 0
    foreach pair $pairList {
      set c0 [lindex $pair 0]
      set c1 [lindex $pair 1]
      set tmp [pw::Connector join -resetDistribution -reject rejectVar [list $c0 $c1]]
      set nreject [llength $rejectVar]
      if { $nreject > 0 } {
        puts "Connectors [$c0 getName] and [$c1 getName] not joined."
      } else {
        incr Joined 1
        set rflag 1
      }
      unset tmp
    }
    puts "Connector pairs joined = $Joined"

    unset pairList
    unset nodeList

  }

  puts "Resetting all connectors to equal spacing."
  set conList [pw::Grid getAll -type pw::Connector]
  foreach con $conList {

    $con replaceDistribution 1 [pw::DistributionTanh create]

  }
  unset conList

  return $rflag
}

proc processBaffleIntersections { tol } {

  #
  #  Baffle domains are labeled on the geometry.
  #  Domains that intersect with baffles domains are also labeled.
  #  Connectors from baffles are inserted into baffle intersect domains.
  #  This assumes open-ended connector chains. No closed-loop connectors are permitted.
  #  It also assumes each connector in the intersected domain is adjacent to only 
  #    one or two other connectors. In other words, no endpoints connected to more
  #    than two connectors.
  #

  # return flag equals zero for success.
  # positive = number of unprocessed connectors

  set rflag 0

  set domList [pw::Grid getAll -type pw::DomainUnstructured]

  set bdomList {}
  set idomList {}
  foreach dom $domList {
    set label [domAttributeFromGeometry $dom "PW:Baffle"]
    if { "Baffle" == $label } {
      lappend bdomList $dom
    }
    if { "Intersect" == $label } {
      lappend idomList $dom
    }
  }

  set numIntersects [llength $idomList]
  set numBaffles [llength $bdomList]

  if {0 < $numIntersects && 0 < $numBaffles } {
    puts "Number of Baffle domains = $numBaffles."
    puts "Number of Baffle intersect domains = $numIntersects."

    # process all intersected domains
    foreach idom $idomList {

      set dbEnts [$idom getDatabaseEntities]

      # create list of connectors on the domain
      set iconList {}
      set saveList {}

      # test connectors of baffle domains
      foreach bdom $bdomList {

        set ne [$bdom getEdgeCount]

        # include connectors that are in the domain using closest function for DB quilt
        for { set i 1 } { $i <= $ne } { incr i } {
          set e [$bdom getEdge $i]
          set nc [$e getConnectorCount]

          for { set j 1 } { $j <= $nc } { incr j } {

            set con [$e getConnector $j]
            set dim [$con getDimension]

            set flag 1
            for { set k 1 } { $k <= $dim && 1 == $flag } { incr k } {
              set pt [$con getXYZ -grid $k]

              set dmin 1.0e20
              foreach dbEnt $dbEnts {
                if { "Quilt" == [$dbEnt getDescription] } {
                  set spt [$dbEnt closestPoint -distance ds $pt]
                  if { $ds < $dmin } {
                    set dmin $ds
                  }
                }
              }
              if { $dmin > $tol } {
                set flag 0
              }
            }
            if { 1 == $flag } {
              lappend iconList $con
              lappend saveList $con
            }
          }
        }
      }

      if { 0 < [llength $iconList] } {

        puts "Number of saved connectors = [llength $iconList]"

        set rflag 0
        foreach con $iconList {
          set n0 [$con getNode Begin]
          set n1 [$con getNode End]
          set c0 [$n0 getConnectors]
          set c1 [$n1 getConnectors]
          set count0 0
          foreach con $c0 {
            set j [lsearch $saveList $con]
            if { -1 == $j } {
              incr count0
            }
          }
          set count1 0
          foreach con $c1 {
            set j [lsearch $saveList $con]
            if { -1 == $j } {
              incr count1
            }
          }

          if { 2 < $count0 || 2 < $count1 } {
            incr rflag 1
          }
        }

        if { 0 < $rflag } {
          puts "Saved connector set is invalid. Number of connectors with more than two neighbors = $rflag"

          unset iconList
          unset saveList
          unset bdomList
          unset idomList
          unset domList
          unset dbEnts

          return $rflag
        }

        # add save connectors to domain as new internal edges
        set bmode [pw::Application begin Modify -notopology $idom]

          set eList {}
          set ne [$idom getEdgeCount]
          for { set i 1 } { $i <= $ne } { incr i } {
            set e [$idom getEdge $i]
            lappend eList $e
          }

          $idom removeEdges -preserve

          # add original back in domain
          foreach ed $eList {
            $idom addEdge $ed
          }
          unset eList

          set flag 1

          while { 1 == $flag } {

            # process connectors as open-ended chains
            set chain {}

            set next ""

            # find first segment with open end
            for { set i 0 } { $i < [llength $iconList] } { incr i } {
              set con [lindex $iconList $i]
              set n0 [$con getNode Begin]
              set n1 [$con getNode End]
              set c0 [$n0 getConnectors]
              set c1 [$n1 getConnectors]
              set count0 0
              foreach tcon $c0 {
                set j [lsearch $saveList $tcon]
                if { -1 != $j } {
                  incr count0 1
                }
              }
              set count1 0
              foreach tcon $c1 {
                set j [lsearch $saveList $tcon]
                if { -1 != $j } {
                  incr count1 1
                }
              }
              if { 1 == $count0 || 1 == $count1 } {
                lappend chain $con
                # remove connector from old list
                set iconList [lremove $iconList $con]
                if { 2 == $count0 } {
                  set next $n0
                }
                if { 2 == $count1 } {
                  set next $n1
                }
                break
              }
              unset c0
              unset c1
              unset n0
              unset n1
            }

            while { "" != $next } {

              foreach con $iconList {
                set n0 [$con getNode Begin]
                set n1 [$con getNode End]
                if { $next == $n0 || $next == $n1 } {
                  set c0 [$n0 getConnectors]
                  set c1 [$n1 getConnectors]
                  set count0 0
                  foreach tcon $c0 {
                    set j [lsearch $saveList $tcon]
                    if { -1 != $j } {
                      incr count0 1
                    }
                  }
                  set count1 0
                  foreach tcon $c1 {
                    set j [lsearch $saveList $tcon]
                    if { -1 != $j } {
                      incr count1 1
                    }
                  }

                  lappend chain $con
                  # remove connector from old list
                  set iconList [lremove $iconList $con]

                  if { $next == $n0 && 2 == $count1 } {
                    set next $n1
                  } elseif { $next == $n1 && 2 == $count0 } {
                    set next $n0
                  } else {
                    set next ""
                  }
                  break;
                }
              }
            }

            set flag 0
            # create new edge
            if { 0 < [llength $chain] } {
              set ed [pw::Edge create]
              # forward pass
              for { set i 0 } { $i < [llength $chain] } { incr i } {
                set con [lindex $chain $i]
                $ed addConnector $con
              }
              # revers pass
              set j [llength $chain]
              for { set i 0 } { $i < [llength $chain] } { incr i } {
                set j [expr $j - 1]
                set con [lindex $chain $j]
                $ed addConnector $con
              }

              # add the edge to the domain
              $idom addEdge $ed
              unset ed
              set flag 1
            }
            unset chain
          }

        $bmode end
        unset bmode

      }

      incr rflag [llength $iconList]
      unset iconList
      unset saveList
      unset dbEnts
    }

  }

  unset bdomList
  unset idomList
  unset domList

  return $rflag
}

# END SCRIPT

# DISCLAIMER:
# TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, POINTWISE DISCLAIMS
# ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED
# TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE, WITH REGARD TO THIS SCRIPT.  TO THE MAXIMUM EXTENT PERMITTED 
# BY APPLICABLE LAW, IN NO EVENT SHALL POINTWISE BE LIABLE TO ANY PARTY 
# FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES 
# WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF 
# BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS) ARISING OUT OF THE 
# USE OF OR INABILITY TO USE THIS SCRIPT EVEN IF POINTWISE HAS BEEN 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND REGARDLESS OF THE 
# FAULT OR NEGLIGENCE OF POINTWISE.
#
