#!/bin/bash +x

# function to run a case that is expected to work
expectSuccess ()
{
  tmp=$2
  filebase=${tmp%.*}
  set -x
  if ( time $ESP_ROOT/bin/serveCSM -batch -verify -outLevel 0 $DICTFLAG $1 ); then
    set +x
    printf "%-30s passed (as expected)\n" $filebase
  else
    set +x
    printf "%-30s failed (as NOT expected)\n" $filebase
  fi
}

# function to run a case that is expected to fail to check error handling
expectErrorHandling ()
{
  tmp=$2
  filebase=${tmp%.*}
  set -x
  if ( time $ESP_ROOT/bin/serveCSM -batch -verify -outLevel 0 $DICTFLAG $1 ); then
    set +x
    printf "%-30s passed (as NOT expected)\n" $filebase
  else
    set +x
    printf "%-30s failed (as expected)\n" $filebase
  fi
}

# function to run a case that is expected to fail due to known errors in OCC
expectOCCFailure ()
{
  tmp=$2
  filebase=${tmp%.*}
  set -x
  if ( time $ESP_ROOT/bin/serveCSM -batch -verify -outLevel 0 $DICTFLAG $1 ); then
    set +x
    printf "%-30s passed (as NOT expected)\n" $filebase
  else
    set +x
    printf "%-30s failed (known OCC failure)\n" $filebase
  fi
}

CSMFILES=`cat csmFiles.txt`
CSMERRORSHANDLING=`cat csmErrorHandling.txt`
CSMOCCERRORS=`cat csmOCCFailure.txt`

echo "Testing CSM files:"
echo "$CSMFILES"
for i in $CSMFILES
do
  filename=data${i##*data}
  if [[ "$CSMERRORSHANDLING" == *"$filename"* ]]; then
    echo $filename | tee -a csmErrorlog.txt
    expectErrorHandling $i $filename | tee -a csmErrorlog.txt
  elif [[ "$CSMOCCERRORS" == *"$filename"* ]]; then
    echo $filename | tee -a csmlog.txt
    expectOCCFailure $i $filename | tee -a csmlog.txt
  else
    echo $filename | tee -a csmlog.txt
    expectSuccess $i $filename | tee -a csmlog.txt
  fi
  echo "--------------------------------------------------------------------------------------------------"
done
