#ifndef AIMU_H
#define AIMU_H
/*
 *      CAPS: Computational Aircraft Prototype Syntheses
 *
 *             AIM Utility Function Prototypes
 *
 *      Copyright 2014-2018, Massachusetts Institute of Technology
 *      Licensed under The GNU Lesser General Public License, version 2.1
 *      See http://www.opensource.org/licenses/lgpl-2.1.php
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "capsTypes.h"

#ifdef __ProtoExt__
#undef __ProtoExt__
#endif
#ifdef __cplusplus
extern "C" {
#define __ProtoExt__
#else
#define __ProtoExt__ extern
#endif

__ProtoExt__ int
  aim_getBodies( void *aimInfo, int *nBody, ego **bodies );
  
__ProtoExt__ int
  aim_newGeometry( void *aimInfo );
  
__ProtoExt__ int
  aim_getUnitSys( void *aimInfo, char **unitSys );

__ProtoExt__ int
  aim_convert( void *aimInfo, const char  *inUnits, double   inValue,
                              const char *outUnits, double *outValue );
  
__ProtoExt__ int
  aim_getIndex( void *aimInfo, /*@null@*/ const char *name,
                enum capssType subtype );
  
__ProtoExt__ int
  aim_getValue( void *aimInfo, int index, enum capssType subtype,
                capsValue ** value );
  
__ProtoExt__ int
  aim_getName( void *aimInfo, int index, enum capssType subtype,
               const char **name );
  
__ProtoExt__ int
  aim_getData( void *aimInfo, const char *name, enum capsvType *vtype,
               int *rank, int *nrow, int *ncol, void **data, char **units );
  
__ProtoExt__ int
  aim_link( void *aimInfo, const char *name, enum capssType stype,
           capsValue *val );
  
__ProtoExt__ int
  aim_setTess( void *aimInfo, ego tess );
  
__ProtoExt__ int
  aim_getDiscr( void *aimInfo, const char *tname, capsDiscr **discr );
  
__ProtoExt__ int
  aim_getDataSet( capsDiscr *discr, const char *dname, enum capsdMethod *method,
                  int *npts, int *rank, double **data );

__ProtoExt__ int
  aim_getBounds( void *aimInfo, int *nTname, char ***tnames );
  
__ProtoExt__ int
  aim_setSensitivity( void *aimInfo, const char *GIname, int irow, int icol );
  
__ProtoExt__ int
  aim_getSensitivity( void *aimInfo, ego body, int ttype, int index, int *npts,
                      double **dxyz );

__ProtoExt__ int
  aim_sensitivity( void *aimInfo, const char *GIname, int irow, int icol,
                   ego tess, int *npts, double **dxyz );

#ifdef __cplusplus
}
#endif

#endif
