## [importPrint]
from __future__ import print_function
## [importPrint]

# Import pyCAPS and os module
## [import]
try:
    import pyCAPS
except:
    print ("Unable to import pyCAPS module")
    raise SystemError
try:
    import os
except:
    print ("Unable to import os module")
    raise SystemError
## [import]

# -----------------------------------------------------------------
# Initialize capsProblem object
# -----------------------------------------------------------------
## [initateProblem]
myProblem = pyCAPS.capsProblem()
## [initateProblem]

# -----------------------------------------------------------------
# Load CSM file and Change a design parameter - area in the geometry 
# Any despmtr from the avlWing.csm file are available inside the pyCAPS script
# They are: thick, camber, area, aspect, taper, sweep, washout, dihedral
# -----------------------------------------------------------------

## [geometry]
myGeometry = myProblem.loadCAPS("./csmData/cfd_airfoilSection.csm")
## [geometry]

# -----------------------------------------------------------------
# Load desired aim 
# -----------------------------------------------------------------
print ("Loading AIM")
## [loadAIM]
myAnalysis = myProblem.loadAIM(aim = "cart3dAIM",
                               analysisDir = "cart3d",
                               capsIntent = "CFD")

## [loadAIM]
# -----------------------------------------------------------------
# Also available are all aimInput values
# Set new Mach/Alt parameters
# -----------------------------------------------------------------

## [setInputs]
myAnalysis.setAnalysisVal("Mach", 0.95)
myAnalysis.setAnalysisVal("alpha", 2.0)
myAnalysis.setAnalysisVal("maxCycles", 1000)
myAnalysis.setAnalysisVal("nDiv", 6)
myAnalysis.setAnalysisVal("maxR", 12)


## [setInputs]

# -----------------------------------------------------------------
# Run AIM pre-analysis
# -----------------------------------------------------------------
## [preAnalysis]
myAnalysis.aimPreAnalysis()
## [preAnalysis]

# -----------------------------------------------------------------
# Run AVL
# -----------------------------------------------------------------
## [runAVL]
print ("Running CART3D")
currentDirectory = os.getcwd() # Get our current working directory 
os.chdir(myAnalysis.analysisDir) # Move into test directory
os.system("flowCart -v -T");
os.chdir(currentDirectory) # Move back to working directory 
## [runAVL]

# -----------------------------------------------------------------
# Run AIM post-analysis
# -----------------------------------------------------------------
## [postAnalysis]
myAnalysis.aimPostAnalysis()
## [postAnalysis]


print ("C_A  " + str(myAnalysis.getAnalysisOutVal("C_A")))
print ("C_Y  " + str(myAnalysis.getAnalysisOutVal("C_Y")))
print ("C_N  " + str(myAnalysis.getAnalysisOutVal("C_N")))
print ("C_D  " + str(myAnalysis.getAnalysisOutVal("C_D")))
print ("C_S  " + str(myAnalysis.getAnalysisOutVal("C_S")))
print ("C_L  " + str(myAnalysis.getAnalysisOutVal("C_L")))
print ("C_l  " + str(myAnalysis.getAnalysisOutVal("C_l")))
print ("C_m  " + str(myAnalysis.getAnalysisOutVal("C_m")))
print ("C_n  " + str(myAnalysis.getAnalysisOutVal("C_n")))
print ("C_M_x  " + str(myAnalysis.getAnalysisOutVal("C_M_x")))
print ("C_M_y  " + str(myAnalysis.getAnalysisOutVal("C_M_y")))
print ("C_M_z  " + str(myAnalysis.getAnalysisOutVal("C_M_z")))

## [output]
# -----------------------------------------------------------------
# Close CAPS 
# -----------------------------------------------------------------
myProblem.closeCAPS()