from __future__ import print_function

# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
import os

# Initialize capsProblem object
myProblem = capsProblem()

# Load CSM file 
myProblem.loadCAPS("./csmData/feaWingBEM.csm")

# Load nastran aim 
myAnalysis = myProblem.loadAIM(aim = "nastranAIM", 
                               altName = "nastran", 
                               analysisDir= "NastranModalWingBEM", 
                               capsIntent = "STRUCTURE")

projectName = "pyCAPS_nastran_Test"
myAnalysis.setAnalysisVal("Proj_Name", projectName) 

myAnalysis.setAnalysisVal("Edge_Point_Max", 6)

myAnalysis.setAnalysisVal("Quad_Mesh", True)

myAnalysis.setAnalysisVal("File_Format", "Free")

myAnalysis.setAnalysisVal("Mesh_File_Format", "Large")

# Set analysis 
eigen = { "extractionMethod"     : "Lanczos", 
          "frequencyRange"       : [0, 10000], 
          "numEstEigenvalue"     : 1,
          "numDesiredEigenvalue" : 10, 
          "eigenNormaliztion"    : "MASS"}
myAnalysis.setAnalysisVal("Analysis", ("EigenAnalysis", eigen))

# Set materials
unobtainium  = {"youngModulus" : 2.2E6 , 
                "poissonRatio" : .5, 
                "density"      : 7850}

madeupium    = {"materialType" : "isotropic", 
                "youngModulus" : 1.2E5 , 
                "poissonRatio" : .5, 
                "density"      : 7850}
myAnalysis.setAnalysisVal("Material", [("Unobtainium", unobtainium),
                                       ("Madeupium", madeupium)])

# Set property 
shell  = {"propertyType"      : "Shell", 
          "material"          : "unobtainium", 
	      "bendingInertiaRatio" : 1.0, # Default - not necesssary           
          "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
          "membraneThickness" : 0.2}
 
myAnalysis.setAnalysisVal("Property", ("Ribs_and_Spars", shell))

# Set constraints 
constraint = {"groupName" : ["Rib_Constraint"], 
              "dofConstraint" : 123456}

myAnalysis.setAnalysisVal("Constraint", ("ribConstraint", constraint))


# Run AIM pre-analysis
myAnalysis.aimPreAnalysis()

####### Run Nastran####################
print ("\n\nRunning Nastran......")  
currentDirectory = os.getcwd() # Get our current working directory 

os.chdir(myAnalysis.analysisDir) # Move into test directory
 
os.system("nastran old=no notify=no batch=no scr=yes sdirectory=./ " + projectName +  ".dat"); # Run Nastran via system call

os.chdir(currentDirectory) # Move back to working directory 
print ("Done running Nastran!")

# Run AIM post-analysis
myAnalysis.aimPostAnalysis()

# Get Eigen-frequencies
print ("\nGetting results for natural frequencies.....")
natrualFreq = myAnalysis.getAnalysisOutVal("EigenFrequency")

mode = 1
for i in natrualFreq:
    print ("Natural freq (Mode {:d}) = ".format(mode) + '{:.5f} '.format(i) + "(Hz)")
    mode += 1    
 
# Close CAPS 
myProblem.closeCAPS()
