## [importPrint]
from __future__ import print_function
## [importPrint]

## [import]
# Import pyCAPS class file
from pyCAPS import capsProblem

# Import os module   
try: 
    import os
except:
    print ("Unable to import os module")
    raise SystemError
## [import]

## [initateProblem]
# Initialize capsProblem object
myProblem = capsProblem()
## [initateProblem]

## [geometry]
# Load CSM file 
myProblem.loadCAPS("./csmData/feaThreeBar.csm")
## [geometry]

## [loadAIM]
# Load nastran aim 
nastranAIM = myProblem.loadAIM(aim = "nastranAIM", 
                               altName = "nastran", 
                               analysisDir= "NastranThreeBar", 
                               capsIntent = "STRUCTURE")
## [loadAIM]

## [setInputs]
# Set project name so a mesh file is generated 
projectName = "threebar_nastran_Test"
nastranAIM.setAnalysisVal("Proj_Name", projectName) 
nastranAIM.setAnalysisVal("File_Format", "Free")
nastranAIM.setAnalysisVal("Mesh_File_Format", "Large")
nastranAIM.setAnalysisVal("Edge_Point_Max", 2);
nastranAIM.setAnalysisVal("Edge_Point_Min", 2);
nastranAIM.setAnalysisVal("Analysis_Type", "Static");
## [setInputs]

## [defineMaterials]
madeupium    = {"materialType" : "isotropic", 
                "youngModulus" : 1.0E7 , 
                "poissonRatio" : .33, 
                "density"      : 0.1}  
                   
nastranAIM.setAnalysisVal("Material", [("Madeupium", madeupium)])
## [defineMaterials]

## [defineProperties] 
rod  =   {"propertyType"      : "Rod", 
		  "material"          : "Madeupium", 
		  "crossSecArea"      : 1.0}

rod2  =   {"propertyType"     : "Rod", 
		  "material"          : "Madeupium", 
		  "crossSecArea"      : 2.0}
		  
nastranAIM.setAnalysisVal("Property", [("bar1", rod),
									   ("bar2", rod2),
									   ("bar3", rod)])
## [defineProperties]                                                          

# Set constraints
## [defineConstraints] 
constraints = []

constraint = {"groupName" 		: ["boundary"], 
              "dofConstraint" 	: 123456}
tmp = ( "conOne", constraint )              
constraints.append( tmp )              
              
constraint = {"groupName" 		: ["boundary"], 
              "dofConstraint" 	: 123}    
tmp = ( "conTwo", constraint )                        
constraints.append( tmp )

nastranAIM.setAnalysisVal("Constraint", constraints)
## [defineConstraints]

## [defineLoad]
loads = []

load = {"groupName" 		: "force", 
	    "loadType" 			: "GridForce", 
	    "forceScaleFactor" 	: 20000.0,
	    "directionVector" 	: [0.8, -0.6, 0.0]}
loads.append( ("loadOne", load) )   
        
load = {"groupName" 		: "force", 
		"loadType" 			: "GridForce", 
		"forceScaleFactor" 	: 20000.0,
		"directionVector" 	: [-0.8, -0.6, 0.0]}        
loads.append( ("loadTwo", load) )        
   
nastranAIM.setAnalysisVal("Load", loads)
## [defineLoad]

## [defineAnalysis]
analysisCases = []

value = {"analysisType" 		: "Static", 
		 "analysisConstraint" 	: "conOne",
         "analysisLoad" 		: "loadOne"}
analysisCases.append( ("analysisOne", value ) )       
       
value = {"analysisType" 		: "Static", 
		 "analysisConstraint" 	: "conTwo",
         "analysisLoad" 		: "loadTwo"}             
analysisCases.append( ("analysisTwo", value ) )

myProblem.analysis["nastran"].setAnalysisVal("Analysis", analysisCases)
## [defineAnalysis]

# Run AIM pre-analysis
## [preAnalysis]
nastranAIM.aimPreAnalysis()
## [preAnalysis]

## [run]
print ("\n\nRunning Nastran......")  
currentDirectory = os.getcwd() # Get our current working directory 
os.chdir(nastranAIM.analysisDir) # Move into test directory
os.system("nastran old=no notify=no batch=no scr=yes sdirectory=./ " + projectName +  ".dat"); # Run Nastran via system call
os.chdir(currentDirectory) # Move back to working directory 
print ("Done running Nastran!")
## [run]

## [postAnalysis]
nastranAIM.aimPostAnalysis() 
## [postAnalysis]

# Close CAPS 
## [close]
myProblem.closeCAPS()
## [close]
