from __future__ import print_function

# Import pyCAPS class file
from pyCAPS import capsProblem

# Import argparse module   
try: 
    import argparse
except: 
    print ("Unable to import argparse module")
    raise ImportError

import os

# Setup and read command line options. Please note that this isn't required for pyCAPS
parser = argparse.ArgumentParser(description = 'Pointwise Pytest Example',
                                 prog = 'pointwise_PyTest',
                                 formatter_class = argparse.ArgumentDefaultsHelpFormatter)

#Setup the available commandline options
parser.add_argument('-workDir', default = "./", nargs=1, type=str, help = 'Set working/run directory')
args = parser.parse_args()

# Initialize capsProblem object
myProblem = capsProblem()

# Load CSM file 
myProblem.loadCAPS("./csmData/cfdMultiBody.csm")

# Load AFLR4 aim 
pointwise = myProblem.loadAIM(aim = "pointwiseAIM", 
                              analysisDir=  str(args.workDir[0]) + "/PointwiseAnalysisTest", 
                              capsIntent = "CFD")

#  COMMENTED code is not implemented yet
# Set project name so a mesh file is generated 
# pointwise.setAnalysisVal("Proj_Name", "pyCAPS_Pointwise_Test") 
# 
# pointwise.setAnalysisVal("Mesh_ASCII_Flag", True)
# 
# pointwise.setAnalysisVal("Mesh_Format", "AFLR3")
# 
# pointwise.setAnalysisVal("Mesh_Quiet_Flag", False)

# useGlobal = False
# 
# if useGlobal:
#     pointwise.setAnalysisVal("BL_Initial_Spacing", 0.1)
#     pointwise.setAnalysisVal("BL_Thickness", 1.0)
#     
# else:
#     inviscidBC = {"boundaryLayerThickness" : 0.0, "boundaryLayerSpacing" : 0.0}
#     viscousBC  = {"boundaryLayerThickness" : 5.0, "boundaryLayerSpacing" : 0.05}
#     
#     # Set mesh sizing parmeters
#     pointwise.setAnalysisVal("Mesh_Sizing", [("Wing1", inviscidBC),
#                                                                ("Wing2", viscousBC)])


# Run AIM pre-analysis
pointwise.aimPreAnalysis()

# Pointwise batch execution notes (pointwise_home_path referes to the Pointwise installation directory):
# Windows (DOS prompt) - 
#     pointwise_home_path\win32\bin\tclsh AutoMeshBatch.glf caps.nmb capsInput.txt
# Linux (terminal) -  
#     pointwise_home_path/pointwise -b AutoMeshBatch.glf caps.nmb capsInput.txt
# Mac OSX (terminal) [tclsh may have a version number, e.g tclsh8.5] - 
#     pointwise_home_path/macosx/Pointwise.app/Contents/MacOS/tclsh AutoMeshBatch.glf caps.nmb capsInput.txt

# Run pointwise (currently setup for Linux execution)
currentDir = os.getcwd()
os.chdir(pointwise.analysisDir)

os.system("pointwise AutoMeshBatch.glf caps.nmb capsInput.txt ")

os.chdir(currentDir)

# Run AIM post-analysis
pointwise.aimPostAnalysis()

# Close CAPS 
myProblem.closeCAPS()