
expectPythonSuccess ()
{
    if ( python $1 -workDir=$2 $3 ); then
    set +x
    echo
    echo "============================================="
    echo "CAPS verification (using pyCAPS) case $1 passed (as expected)"
    echo "============================================="
    echo
    else
    set +x
    echo
    echo "================================================="
    echo "CAPS verification (using pyCAPS) case $1 failed (as NOT expected)"
    echo "================================================="
    echo
    rm -fr $2 csmData
    exit 1
    fi
    set -x
}

# function to run a case that is expected to fail
expectPythonFailure ()
{
    if ( python $1 -workDir=$2 $3); then
    set +x
    echo
    echo "================================================="
    echo "CAPS verification (using pyCAPS) case $1 passed (as NOT expected)"
    echo "================================================="
    echo
    rm -fr $2 csmData
    exit 1
    else
    set +x
    echo
    echo "============================================="
    echo "CAPS verification (using pyCAPS) case $1 failed (as expected)"
    echo "============================================="
    echo
    
    fi
    set -x
}

# Create/append PYTHONPATH for pyCAPS.so 
export PYTHONPATH=$PYTHONPATH:$ESP_ROOT/lib

# Copy csmData folder - pyTest examples are hardcoded to look for ./csmData
cp -r ../csmData ./

export capsPythonRegDir=runDirectory

mkdir $capsPythonRegDir

######### Execute pyTest examples ########

###### Meshing examples #########
if [ "$AFLR" != "" ]; then
    
    expectPythonSuccess "../aflr3_PyTest.py" $capsPythonRegDir
    expectPythonSuccess "../aflr4_PyTest.py" $capsPythonRegDir

    if [ "$TETGEN" != "" ]; then
        expectPythonSuccess "../aflr4_and_Tetgen_PyTest.py" $capsPythonRegDir
    fi
fi

if [ "$TETGEN" != "" ]; then
    expectPythonSuccess "../tetgen_PyTest.py" $capsPythonRegDir
fi

mkdir ./${capsPythonRegDir}/DelaundoAnalysisTest # Delaundo example requires a datafile 
cp datafiles/delaundoMesh.mesh ./${capsPythonRegDir}/DelaundoAnalysisTest/
expectPythonSuccess "../delaundo_PyTest.py" $capsPythonRegDir "-noAnalysis"


###### FUN3D examples  ##########
#expectPythonSuccess "../fun3d_PyTest.py" $capsPythonRegDir

if [ "$AFLR" != "" ]; then

    mkdir ./${capsPythonRegDir}/FUN3DandAFRLMeshTest # FUN3D - AFLR example requires a datafile 
    cp datafiles/fun3dAFLRTest.forces ./${capsPythonRegDir}/FUN3DandAFRLMeshTest/

    expectPythonSuccess "../fun3d_and_AFLR4_AFLR3_PyTest.py" $capsPythonRegDir "-noAnalysis"
fi 

if [ "$TETGEN" != "" ]; then
    
    mkdir ./${capsPythonRegDir}/FUN3DTetgenAnalysisTest # FUN3D - Tetgen example requires a datafile 
    cp datafiles/fun3dTetgenTest.forces ./${capsPythonRegDir}/FUN3DTetgenAnalysisTest/
    
    expectPythonSuccess "../fun3d_and_Tetgen_PyTest.py" $capsPythonRegDir "-noAnalysis"
fi

###### Mystran examples  ##########
mkdir ./${capsPythonRegDir}/MystranModalWingBEM # Mystran basic example requires a datafile 
cp datafiles/MystranModalWingBEM.OP1 ./${capsPythonRegDir}/MystranModalWingBEM/
expectPythonSuccess "../mystran_PyTest.py" $capsPythonRegDir "-noAnalysis"

mkdir ./${capsPythonRegDir}/MystranModalAGARD445 # Mystran AGARD445 example requires a datafile 
cp datafiles/MystranModalAGARD445.OP1 ./${capsPythonRegDir}/MystranModalAGARD445/
    
expectPythonSuccess "../mystran_AGARD445_PyTest.py" $capsPythonRegDir "-noAnalysis"

###### XFoil examples ##########
mkdir ./${capsPythonRegDir}/xFoilAnalysisTest # XFoil basic example requires a datafile 
cp datafiles/xfoilPolar.dat ./${capsPythonRegDir}/xFoilAnalysisTest/
expectPythonSuccess "../xfoil_PyTest.py" $capsPythonRegDir "-noAnalysis" 

###### FRICTION examples ##########
mkdir ./${capsPythonRegDir}/FrictionAnalysisTest # Friction basic example requires a datafile 
cp datafiles/frictionOutput.txt ./${capsPythonRegDir}/FrictionAnalysisTest/
expectPythonSuccess "../friction_PyTest.py" $capsPythonRegDir "-noAnalysis" 

#################################

# Cleanup 
rm -fr $capsPythonRegDir csmData
