# Execute pyTest scripts - actually execute the analysis code 

expectPythonSuccess ()
{
    if ( $pythonVer $1); then
    set +x
    echo
    echo "============================================="
    echo "CAPS verification (using pyCAPS) case $1 passed (as expected)"
    echo "============================================="
    echo
    else
    set +x
    echo
    echo "================================================="
    echo "CAPS verification (using pyCAPS) case $1 failed (as NOT expected)"
    echo "================================================="
    echo
    cd ../
    rm -fr $2 
    exit 1
    fi
    set -x
}

# function to run a case that is expected to fail
expectPythonFailure ()
{
    if ( $pythonVer $1); then
    set +x
    echo
    echo "================================================="
    echo "CAPS verification (using pyCAPS) case $1 passed (as NOT expected)"
    echo "================================================="
    echo
     cd ../
    rm -fr $2 
    exit 1
    else
    set +x
    echo
    echo "============================================="
    echo "CAPS verification (using pyCAPS) case $1 failed (as expected)"
    echo "============================================="
    echo
    
    fi
    set -x
}

export pythonVer="python2.7"

export capsPyTest=capsPyTest

# Make work directory 
mkdir $capsPyTest

# cd into directory and create symbolic link to 
cd $capsPyTest
ln -s ../csmData

######### Execute pyTest examples ########

##### LinearAero examples ###########
expectPythonSuccess "../awave_PyTest.py" $capsPyTest
expectPythonSuccess "../avl_PyTest.py" $capsPyTest
expectPythonSuccess "../avl_ControlSurf_PyTest.py" $capsPyTest
expectPythonSuccess "../friction_PyTest.py" $capsPyTest
expectPythonSuccess "../xfoil_PyTest.py" $capsPyTest
expectPythonSuccess "../tsfoil_PyTest.py" $capsPyTest

###### Meshing examples #########
expectPythonSuccess "../aflr3_PyTest.py" $capsPyTest
expectPythonSuccess "../aflr4_PyTest.py" $capsPyTest
 
expectPythonSuccess "../aflr4_and_Tetgen_PyTest.py" $capsPyTest

expectPythonSuccess "../tetgen_PyTest.py" $capsPyTest

expectPythonSuccess "../delaundo_PyTest.py" $capsPyTest

expectPythonSuccess "../egadsTess_PyTest.py" $capsPyTest

###### FUN3D examples  ##########
expectPythonSuccess "../fun3d_PyTest.py" $capsPyTest
expectPythonSuccess "../fun3d_and_AFLR4_AFLR3_PyTest.py" $capsPyTest
expectPythonSuccess "../fun3d_and_AFLR4_NodeDist_PyTest.py" $capsPyTest
expectPythonSuccess "../fun3d_and_AFLR4_PyTest.py" $capsPyTest
expectPythonSuccess "../fun3d_and_AFLR4_PyTest.py" $capsPyTest
expectPythonSuccess "../fun3d_and_egadsTess_PyTest.py" $capsPyTest
expectPythonSuccess "../fun3d_and_Tetgen_PyTest.py" $capsPyTest
expectPythonSuccess "../fun3d_and_Delaundo_PyTest.py" $capsPyTest

###### SU2 examples  ##########
expectPythonSuccess "../su2_and_AFLR3_PyTest.py" $capsPyTest
expectPythonSuccess "../su2_and_AFLR4_PyTest.py" $capsPyTest
expectPythonSuccess "../su2_and_Tetgen_PyTest.py" $capsPyTest

###### Mystran examples  ##########
expectPythonSuccess "../mystran_PyTest.py" $capsPyTest
expectPythonSuccess "../mystran_AGARD445_PyTest.py" $capsPyTest
expectPythonSuccess "../mystran_MultiLoadCase_PyTest.py" $capsPyTest

##### Nastran examples #############
expectPythonSuccess "../nastran_PyTest.py" $capsPyTest
expectPythonSuccess "../nastran_SingleLoadCase_PyTest.py" $capsPyTest
expectPythonSuccess "../nastran_MultiLoadCase_PyTest.py" $capsPyTest

expectPythonSuccess "../nastran_OptimizationMultiLoad_PyTest.py" $capsPyTest

expectPythonSuccess "../nastran_ThreeBarDesign_PyTest.py" $capsPyTest
expectPythonSuccess "../nastran_ThreeBarFreq_PyTest.py" $capsPyTest
expectPythonSuccess "../nastran_ThreeBarLinkDesign_PyTest.py" $capsPyTest
expectPythonSuccess "../nastran_ThreeBarMultiLoad_PyTest.py" $capsPyTest

expectPythonSuccess "../nastran_Composite_PyTest.py" $capsPyTest
expectPythonSuccess "../nastran_CompositeWingDesign_PyTest.py" $capsPyTest
expectPythonSuccess "../nastran_CompositeWingFreq_PyTest.py" $capsPyTest

##### Astros examples  #############
expectPythonSuccess "../astros_SingleLoadCase_PyTest.py" $capsPyTest
expectPythonSuccess "../astros_CompositeWingFreq_PyTest.py" $capsPyTest
expectPythonSuccess "../astros_AGARD445_PyTest.py" $capsPyTest

##### Aeroelastic examples #########
expectPythonSuccess "../aeroelasticModal_Fun3D_and_Astros.py" $capsPyTest
expectPythonSuccess "../aeroelasticModal_Fun3D_and_Mystran.py" $capsPyTest
expectPythonSuccess "../aeroelasticModal_Fun3D_and_Nastran.py" $capsPyTest

expectPythonSuccess "../aeroelasticSimple_Displacement_Fun3D_and_Mystran.py" $capsPyTest
expectPythonSuccess "../aeroelasticSimple_Pressure_Fun3D_and_Mystran.py" $capsPyTest
expectPythonSuccess "../aeroelasticSimple_Iterative_Fun3D_and_Mystran.py" $capsPyTest

expectPythonSuccess "../aeroelasticSimple_Displacement_SU2_and_Mystran.py" $capsPyTest
expectPythonSuccess "../aeroelasticSimple_Pressure_SU2_and_Mystran.py" $capsPyTest

expectPythonSuccess "../aeroelastic_Iterative_Fun3D_and_Mystran.py" $capsPyTest

cd ../
# Cleanup 
rm -fr $capsPyTest 