cdef extern from "egadsTypes.h":
    ctypedef struct ego:
        pass
    
    cdef int FACE "FACE"
    cdef int EDGE "EDGE"
    cdef int BODY "BODY"
    
    cdef int MODEL "MODEL"
    
    cdef int NODE "NODE"
    cdef int WIREBODY "WIREBODY"
    cdef int FACEBODY "FACEBODY"
    cdef int SHEETBODY "SHEETBODY"
    cdef int SOLIDBODY "SOLIDBODY"
    
    cdef int ATTRINT  "ATTRINT"
    cdef int ATTRREAL "ATTRREAL"
    cdef int ATTRSTRING "ATTRSTRING"
    cdef int ATTRCSYS   "ATTRCSYS"
            
cdef extern from "egads.h":

    #/* memory functions */

    void *EG_alloc( int nbytes )
    
    void *EG_calloc( int nele, int size )
    
    void *EG_reall( void *ptr,
                    int nbytes )
    
    void EG_free( void *pointer )
    
    #/* base-level object functions */
    
    int  EG_saveModel( const ego model, const char *name )
    
    int  EG_deleteObject( ego object )
    
    int  EG_copyObject( const ego object, void *oform, ego *copy )
    
    #/* attribute functions */
    
    int  EG_attributeNum( const ego obj, int *num )
    
    int  EG_attributeGet( const ego obj, int index, const char **name,
                          int *atype, int *len, 
                          const int    **ints,
                          const double **reals, 
                          const char   **str )
    
    int  EG_attributeRet( const ego obj, const char *name, int *atype, 
                          int *len, 
                          const int    **ints,
                          const double **reals,
                          const char   **str )
    
    #/* topology functions */

    int EG_getTopology( const ego topo, ego *geom, int *oclass, 
                        int *type, double *limits, 
                        int *nChildren, ego **children, int **sense )
                                  
    int EG_makeTopology( ego context, ego geom, int oclass, int mtype, 
                         double *limits, int nChildren, ego *children,
                         int *senses, ego *topo )
    
    int EG_getBodyTopos( const ego body, ego src,
                         int oclass, int *ntopo, ego **topos )
    
    int EG_indexBodyTopo( const ego body, const ego src )
                                   
    int EG_getBoundingBox( const ego topo, double *bbox )
    
    #/* tessellation functions */
    
    int EG_makeTessBody( ego object, double *params, ego *tess )
    
    int EG_getTessFace( const ego tess, int fIndex, int *len, 
                         const double **xyz, const double **uv, 
                         const int **ptype, const int **pindex, 
                         int *ntri, const int **tris, 
                         const int **tric )
    
    int EG_localToGlobal( const ego tess, int index, int local,
                          int *globalID )
    
    int  EG_getGlobal( const ego tess, int globalID, int *ptype,
                       int *pindex, double *xyz )
    
cdef extern from "egadsErrors.h":
    cdef int EGADS_TESSTATE
    cdef int EGADS_EXISTS
    cdef int EGADS_ATTRERR
    cdef int EGADS_TOPOCNT
    cdef int EGADS_OCSEGFLT
    cdef int EGADS_BADSCALE
    cdef int EGADS_NOTORTHO
    cdef int EGADS_DEGEN
    cdef int EGADS_CONSTERR
    cdef int EGADS_TOPOERR
    cdef int EGADS_GEOMERR
    cdef int EGADS_NOTBODY
    cdef int EGADS_WRITERR
    cdef int EGADS_NOTMODEL
    cdef int EGADS_NOLOAD
    cdef int EGADS_RANGERR
    cdef int EGADS_NOTGEOM
    cdef int EGADS_NOTTESS
    cdef int EGADS_EMPTY
    cdef int EGADS_NOTTOPO
    cdef int EGADS_REFERCE
    cdef int EGADS_NOTXFORM
    cdef int EGADS_NOTCNTX
    cdef int EGADS_MIXCNTX
    cdef int EGADS_NODATA
    cdef int EGADS_NONAME
    cdef int EGADS_INDEXERR
    cdef int EGADS_MALLOC
    cdef int EGADS_NOTOBJ
    cdef int EGADS_NULLOBJ
    cdef int EGADS_NOTFOUND
    cdef int EGADS_SUCCESS "EGADS_SUCCESS"
    cdef int EGADS_OUTSIDE