// wv.js implements functions for wv
// written by John Dannenhoffer and Bob Haimes


// Post a message to the bottom frame 
function postMessage(mesg) {
    // alert("in postMessage(" + mesg + ")");

    var botm = document.getElementById("botmframe");

    var text = document.createTextNode(mesg);
    botm.insertBefore(text, botm.lastChild);

    var br = document.createElement("br");
    botm.insertBefore(br, botm.lastChild);

    br.scrollIntoView();
}

function postParseError(text) {
	postMessage(" Server Message: Error parsing " + text);
}

// Send a message to the server
function browserToServer(text) {
    // alert("browserToServer(text="+text+")");

    if (wv.debugUI) {
        var date = new Date;
        console.log("("+date.toTimeString().substring(0,8)+") browser-->server: "+text.substring(0,40));
    }

    wv.socketUt.send(text);
}

function resizeFrames() {

    // get the size of the client (minus 20 to account for possible scrollbars)
    var body = document.getElementById("mainBody");
    var bodyWidth  = body.clientWidth  - 20;
    var bodyHeight = body.clientHeight - 20;

    // get the elements associated with the frames and the canvas
    var left = document.getElementById("leftframe");
    var rite = document.getElementById("riteframe");
    var botm = document.getElementById("botmframe");
    var keyf = document.getElementById("keyframe");
    var canv = document.getElementById(wv.canvasID);
    if (wv.canvasKY != undefined)
      var canf = document.getElementById(wv.canvasKY);

    // compute and set the widths of the frames
    //    (do not make leftframe larger than 250px)
    var leftWidth = Math.round(0.25 * bodyWidth);
    if (leftWidth > 250)   leftWidth = 250;
    var riteWidth = bodyWidth - leftWidth;
    var canvWidth = riteWidth - 20;
    var keyfWidth = leftWidth - 20;

    left.style.width = leftWidth + "px";
    rite.style.width = riteWidth + "px";
    botm.style.width = riteWidth + "px";
    keyf.style.width = leftWidth + "px";
    canv.style.width = canvWidth + "px";
    canv.width       = canvWidth;
    if (wv.canvasKY != undefined) {
      canf.style.width = keyfWidth + "px";
      canf.width       = keyfWidth;
    }

    // compute and set the heights of the frames
    //    (do not make botm frame larger than 200px)
    var botmHeight = Math.round(0.20 * bodyHeight);
    if (botmHeight > 200)   botmHeight = 200;
    var  topHeight = bodyHeight - botmHeight;
    var canvHeight =  topHeight - 25;
    var keyfHeight = botmHeight - 25;

    left.style.height =  topHeight + "px";
    rite.style.height =  topHeight + "px";
    botm.style.height = botmHeight + "px";
    keyf.style.height = botmHeight + "px";
    canv.style.height = canvHeight + "px";
    canv.height       = canvHeight;
    
    if (wv.canvasKY != undefined) {
      canf.style.height = keyfHeight + "px";
      canf.height       = keyfHeight;
      // force a key redraw
      wv.drawKey = 1;
    }
}

//
// Event Handlers
//

function getCursorXY(e)
{
    if (!e) var e = event;

    wv.cursorX  = e.clientX;
    wv.cursorY  = e.clientY;
    wv.cursorX -= wv.offLeft + 1;
    wv.cursorY  = wv.height - wv.cursorY + wv.offTop + 1;

    wv.modifier = 0;
    if (e.shiftKey) wv.modifier |= 1;
    if (e.altKey  ) wv.modifier |= 2;
    if (e.ctrlKey ) wv.modifier |= 4;
}


function getMouseDown(e)
{
    if (!e) var e = event;

    wv.startX   = e.clientX;
    wv.startY   = e.clientY;
    wv.startX  -= wv.offLeft + 1;
    wv.startY   = wv.height - wv.startY + wv.offTop + 1;

    wv.dragging = true;
    wv.button   = e.button;

    wv.modifier = 0;
    if (e.shiftKey) wv.modifier |= 1;
    if (e.altKey  ) wv.modifier |= 2;
    if (e.ctrlKey ) wv.modifier |= 4;
}


function getMouseUp(e)
{
    wv.dragging = false;
}


function mouseLeftCanvas(e)
{
    if (wv.dragging) {
        wv.dragging = false;
    }
}


function getKeyPress(e)
{
    if (!e) var e = event;

    wv.keyPress = e.charCode;
}


//
// call-back functions
//

function wvInitUI()
{

    // set up extra storage for matrix-matrix multiplies
    wv.uiMatrix = new J3DIMatrix4();

                                  // ui cursor variables
    wv.cursorX  = -1;              // current cursor position
    wv.cursorY  = -1;
    wv.keyPress = -1;              // last key pressed
    wv.startX   = -1;              // start of dragging position
    wv.startY   = -1;
    wv.button   = -1;              // button pressed
    wv.modifier =  0;              // modifier (shift,alt,cntl) bitflag
    wv.offTop   =  0;              // offset to upper-left corner of the canvas
    wv.offLeft  =  0;
    wv.dragging = false;           // true during drag operation
    wv.picking  =  0;              // keycode of command that turns picking on
    wv.locating  =  0;             // keycode of command that turned locating on
    wv.focus     = [0, 0, 0, 1];   // focus data needed in locating
    
    wv.gridToggle = true;  	   	   // Keep track if all grid states true = on , false = off
    
    wv.transparenceToggle = true;  // Keep track of transparence states
    
    wv.lowerLimit   = -1;          // lower limit in key
    wv.upperLimit   = +1;          // upper limit in key
    
    wv.keyRange = [-1, 1];         // Initial limit range for the color in the key
    
//  wv.pick                        // set to 1 to turn picking on
//  wv.picked                      // sceneGraph object that was picked
//  wv.locate                      // set to 1 to turn on locating
//  wv.sceneGraph                  // pointer to sceneGraph
//  wv.located                     // coordinates (local) that were pointed at
//  wv.centerV                     // set to 1 to put wv into view centering mode
//  wv.sceneUpd                    // set to 1 when scene needs rendering
//  wv.sgUpdate                    // is 1 if the sceneGraph has been updated
//  wv.socketUt.send(text)         // function to send text to server
//  wv.plotAttrs                   // plot attributes   
 

    var canvas = document.getElementById(wv.canvasID);
    canvas.addEventListener('mousemove',  getCursorXY,     false);
    canvas.addEventListener('mousedown',  getMouseDown,    false);
    canvas.addEventListener('mouseup',    getMouseUp,      false);
    canvas.addEventListener('mouseout',   mouseLeftCanvas, false);
    document.addEventListener('keypress',   getKeyPress,   false);
    
    // Add listener to key
    var keycan = document.getElementById(wv.canvasKY);
    keycan.addEventListener('mouseup',    setKeyLimits,    false);
    
}


function wvUpdateUI()
{

    // special code for delayed-picking mode
    if (wv.picking > 0) {

        // if something is picked, post a message
        if (wv.picked !== undefined) {

        	// second part of '^' operation
            if (wv.picking == 94) {
                var mesg = "Picked: "+wv.picked.gprim;

            //    try {
            //        var attrs = wv.sgData[wv.picked.gprim];
            //        for (var i = 0; i < attrs.length; i+=2) {
            //            mesg = mesg + "\n        "+attrs[i]+"= "+attrs[i+1];
            //        }
            //    } catch (x) {
            //    }
                postMessage(mesg);
            }
            
            // second part of 'q' operation
            if (wv.picking == 113) {
                postMessage("Picked: " + wv.picked.gprim);
                //wv.socketUt.send("Picked: " + wv.picked.gprim);
            }

            wv.picked  = undefined;
            wv.picking = 0;
            wv.pick    = 0;

        // abort picking on mouse motion
        } else if (wv.dragging) {
            postMessage("Picking aborted");

            wv.picking = 0;
            wv.pick    = 0;
        }

        wv.keyPress = -1;
        wv.dragging = false;
    }

    // special code for delayed-locating mode
    if (wv.locating > 0) {
    	// if something is located, post a message
        if (wv.located !== undefined) {

            // second part of '@' operation
            if (wv.locating == 64) {
                var xloc = wv.focus[3] * wv.located[0] + wv.focus[0];
                var yloc = wv.focus[3] * wv.located[1] + wv.focus[1];
                var zloc = wv.focus[3] * wv.located[2] + wv.focus[2];

                postMessage("Located: x="+xloc.toFixed(4)+", y="+yloc.toFixed(4)+
                                   ", z="+zloc.toFixed(4));
            }

            wv.located  = undefined;
            wv.locating = 0;
            wv.locate   = 0;

        // abort locating on mouse motion
        } else if (wv.dragging) {
            postMessage("Locating aborted");

            wv.locating = 0;
            wv.locate   = 0;
        }

        wv.keyPress = -1;
        wv.dragging = false;
    }
    
    // if the tree has not been created but the scene graph (possibly) exists...
    if (wv.sgUpdate == 1 && (wv.sceneGraph !== undefined)) {
    	
    	// ...count number of primitives in the scene graph
        var count = 0;
        
        for (var gprim in wv.sceneGraph) {

            // parse the name
            //var matches = gprim.split(" ");

        	//var ibody = Number(matches[1]);
        	//if        (matches[2] == "Face") {
        	//    var iface = matches[3];
        	//} else if (matches[2] == "Loop") {
        	//    var iloop = matches[3];
        	//} else if (matches[2] == "Edge") {
        	//    var iedge = matches[3];
        	//} else {
        	//   alert("unknown type: " + matches[2]);
        	//    continue;
        	//}

            // determine if Body does not exists
        	//var knode = -1;
        	//for (var jnode = 1; jnode < myTree.name.length; jnode++) {
        	//   if (myTree.name[jnode] == "Body " + ibody) {
        	//       knode = jnode;
        	//   }
        	//}

            // if Body does not exist, create it and its Face, Loop, and Edge
            //    subnodes now
        	// var kface, kloop, kedge;
        	// if (knode < 0) {
        	//    postMessage("Processing Body " + ibody);

        	//    myTree.addNode(0, "Body " + ibody, "*");
        	//   knode = myTree.name.length - 1;

        	//    myTree.addNode(knode, "__Faces", "*");
        	//    kface = myTree.name.length - 1;

            //    myTree.addNode(knode, "__Loops", "*");
        	//    kloop = myTree.name.length - 1;

        	//    myTree.addNode(knode, "__Edges", "*");
        	//    kedge = myTree.name.length - 1;

            // otherwise, get pointers to the face-group and loop-group nodes
        	//} else {
        	//    kface = myTree.child[knode];
        	//    kloop = kface + 1;
        	//   kedge = kloop + 1;
        	//}

            // make the tree node
        	// if        (matches[2] == "Face") {
        	//    myTree.addNode(kface, "____face " + iface, gprim);
        	// } else if (matches[2] == "Loop") {
        	//    myTree.addNode(kloop, "____loop " + iloop, gprim);
        	//            } else if (matches[2] == "Edge") {
        	//    myTree.addNode(kedge, "____edge " + iedge, gprim);
        	// }

        	// count++;
        }

        // if we had any primitives, we are assuming that we have all of
        //    them, so build the tree and remember that we have
        //    built the tree
        if (count > 0) {
            myTree.build();
            wv.sgUpdate = 0;
        }
    }

    // deal with key presses
    if (wv.keyPress != -1) {

        // '?' -- help
        if (wv.keyPress ==  63) {
        	postMessage("Key Options:");
        	postMessage("^ - query object at cursor");
        	postMessage("G or g - toggle grid on and off");
        	//postMessage("T or t - toggle transparence");
            postMessage("N or n - next scalar");
            postMessage("L or l - change scalar limits");
            postMessage("C or c - change color map");
            postMessage("R or r - reverse/invert color map");
            postMessage("q - query at cursor");
            //postMessage("m - set view matrix");
            postMessage("M - current view matrix");
            postMessage("x - view from -X direction");
            postMessage("X - view from +X direction");
            postMessage("y - view from -Y direction");
            postMessage("Y - view from +Y direction");
            postMessage("z - view from -Z direction");
            postMessage("Z - view from +Z direction");
            postMessage("* - center view at cursor");
            postMessage("@ - get coords. at cursor");
            postMessage("? - get help");

        // '^' -- query at cursor
        } else if (wv.keyPress == 94) {
            wv.picking  = 94;
            wv.pick     = 1;
            wv.sceneUpd = 1;
            
        // 'G' or 'g' -- toggle grid on and off
        } else if (wv.keyPress == 71 || wv.keyPress == 103) {
           
        	
        	for (var gprim in wv.sceneGraph) { // Loop through scene primatives
        		
        		// Turn OFF all grid lines 
        		if (wv.gridToggle == true) {
        			
        			wv.sceneGraph[gprim].attrs  &= ~wv.plotAttrs.LINES;
        			wv.sceneGraph[gprim].attrs  &= ~wv.plotAttrs.POINTS;
        			
        		} else { // Turn ON all grid lines 
        			wv.sceneGraph[gprim].attrs |=  wv.plotAttrs.LINES;
        			wv.sceneGraph[gprim].attrs |=  wv.plotAttrs.POINTS;
        		}
        	}
        	
        	if (wv.gridToggle == true) {
        		wv.gridToggle = false;
        	} else {
        		wv.gridToggle = true;
        	}
        	
        	wv.sceneUpd = 1;
        
        // 'T' or 't' -- toggle transparence on and off
        } else if (wv.keyPress == 84 || wv.keyPress == 116) {
        	
        	for (var gprim in wv.sceneGraph) { // Loop through scene primatives
        		
        		// Turn OFF all transparence
        		if (wv.transparenceToggle == true) {
        			
        			wv.sceneGraph[gprim].attrs  &= ~wv.plotAttrs.TRANSPARENT;
        	
        	 	} else { // Turn ON all transparence
        	 		wv.sceneGraph[gprim].attrs |=  wv.plotAttrs.TRANSPARENT;
        			
        	 		}
        	}
        	
        	wv.sceneUpd = 1;
        	
        	if (wv.transparenceToggle == true) {
        		wv.transparenceToggle = false;
        	} else {
        		wv.transparenceToggle = true;
        	}
        	 
        // 'M' -- dump view matrix
        } else if (wv.keyPress == 77) {
            var mVal = wv.mvMatrix.getAsArray();
            postMessage(" View Matrix (scale = " + wv.scale + "):");
            postMessage("  "+mVal[ 0]+" "+mVal[ 1]+" "+mVal[ 2]+" "+mVal[ 3]);
            postMessage("  "+mVal[ 4]+" "+mVal[ 5]+" "+mVal[ 6]+" "+mVal[ 7]);
            postMessage("  "+mVal[ 8]+" "+mVal[ 9]+" "+mVal[10]+" "+mVal[11]);
            postMessage("  "+mVal[12]+" "+mVal[13]+" "+mVal[14]+" "+mVal[15]);
      
        // 'N' or 'n' -- next scalar
        } else if (wv.keyPress == 78 || wv.keyPress == 110) {
        	browserToServer("nextScalar");
        	browserToServer("getRange");
        	
        	//turnOnColor();
        	
         // 'P' or 'p' -- previous scalar
        } else if (wv.keyPress == 80 || wv.keyPress == 112) {
            browserToServer("previousScalar");
            browserToServer("getRange");
            
            //turnOnColor();
            
        // 'L' or 'l' -- Set scalar limits
        } else if (wv.keyPress == 76 || wv.keyPress == 108) {
        	
        	setKeyLimits(null);
        	//turnOnColor();
        	
        // 'C' or 'c' -- Set color map 
        } else if (wv.keyPress == 67 || wv.keyPress == 99) {
        	
        	var cmap = (prompt("Enter desired color map", cmap))
        	if (cmap !=null) {
        	
        		var colorMap  = "colorMap," + cmap.toLowerCase();
        	
        		//alert(colorMap);
        	
        		var levels = (prompt("Enter desired number of contour levels", levels))
            	if (levels !=null) {
            	
            		var contourLevel  = "contourLevel," + parseInt(levels);
            	
            		//alert(contourLevel);
            	
            		browserToServer(contourLevel);
            		//turnOnColor();
            		
            	}
        		
        		browserToServer(colorMap);
        		//turnOnColor();
        	}
        
        	
        // 'm' -- set view matrix
        //} else if (wv.keyPress == 109) {
        	//    var mVal  = wv.mvMatrix.getAsArray();
        	// wv.scale  = parseFloat(prompt("Enter scale", wv.scale));
        	// var strng = prompt("Matrix Pos  0- 3",
        	//                   mVal[ 0]+" "+mVal[ 1]+" "+mVal[ 2]+" "+mVal[ 3]);
        	// var split = strng.split(' ');
        	// var len   = split.length;
        	//if (len > 4) len = 4;
        	//for (var m =  0; m <    len; m++) mVal[m] = parseFloat(split[m   ]);
        	// strng = prompt("Matrix Pos  4- 7",
        	//mVal[ 4]+" "+mVal[ 5]+" "+mVal[ 6]+" "+mVal[ 7]);
        	//split = strng.split(' ');
        	//len   = split.length;
        	//if (len > 4) len = 4;
        	//for (var m =  4; m <  4+len; m++) mVal[m] = parseFloat(split[m- 4]);
        	//strng = prompt("Matrix Pos  8-11",
        	//                   mVal[ 8]+" "+mVal[ 9]+" "+mVal[10]+" "+mVal[11]);
        	//split = strng.split(' ');
        	//len   = split.length;
        	//if (len > 4) len = 4;
        	//for (var m =  8; m <  8+len; m++) mVal[m] = parseFloat(split[m- 8]);
        	//strng = prompt("Matrix Pos 12-15",
        	//                   mVal[12]+" "+mVal[13]+" "+mVal[14]+" "+mVal[15]);
        	//split = strng.split(' ');
        	//len   = split.length;
        	//if (len > 4) len = 4;
        	//for (var m = 12; m < 12+len; m++) mVal[m] = parseFloat(split[m-12]);
        	//wv.mvMatrix.load(mVal);
        	//wv.sceneUpd = 1;
        	
        // 'R' or 'r' -- Reverse/ invert the color map  
        } else if (wv.keyPress == 82 || wv.keyPress == 114) {
        	
        	browserToServer("reverseMap");

        // 'q' -- query at cursor
        } else if (wv.keyPress == 113) {
            wv.picking  = 113;
            wv.pick     = 1;
            wv.sceneUpd = 1;

        // 'x' -- view from -X direction
        } else if (wv.keyPress == 120) {
            wv.mvMatrix.makeIdentity();
            wv.mvMatrix.rotate(+90, 0,1,0);
            wv.sceneUpd = 1;

        // 'X' -- view from +X direction
        } else if (wv.keyPress ==  88) {
            wv.mvMatrix.makeIdentity();
            wv.mvMatrix.rotate(-90, 0,1,0);
            wv.sceneUpd = 1;

        // 'y' -- view from +Y direction
        } else if (wv.keyPress == 121) {
            wv.mvMatrix.makeIdentity();
            wv.mvMatrix.rotate(-90, 1,0,0);
            wv.sceneUpd = 1;

        // 'Y' -- view from +Y direction
        } else if (wv.keyPress ==  89) {
            wv.mvMatrix.makeIdentity();
            wv.mvMatrix.rotate(+90, 1,0,0);
            wv.sceneUpd = 1;

        // 'z' -- view from +Z direction
        } else if (wv.keyPress ==  122) {
            wv.mvMatrix.makeIdentity();
            wv.mvMatrix.rotate(180, 1,0,0);
            wv.sceneUpd = 1;

        // 'Z' -- view from +Z direction
        } else if (wv.keyPress ==  90) {
            wv.mvMatrix.makeIdentity();
            wv.sceneUpd = 1;
          
        // '*' -- center view at cursor
        } else if (wv.keyPress == 42) {
            wv.centerV = 1;
            
        // '@' -- locate at cursor
        } else if (wv.keyPress == 64) {
            wv.locating = 64;
            wv.locate   = 1;

        } else {
            postMessage("'" + String.fromCharCode(wv.keyPress)
                        + "' is not defined.  Use '?' for help.");
        }
    }

    wv.keyPress = -1;

    // UI is in screen coordinates (not object)
    wv.uiMatrix.load(wv.mvMatrix);
    wv.mvMatrix.makeIdentity();

    // deal with mouse movement
    if (wv.dragging) {

        // cntrl is down (rotate)
        if (wv.modifier == 4) {
            var angleX =  (wv.startY - wv.cursorY) / 4.0;
            var angleY = -(wv.startX - wv.cursorX) / 4.0;
            if ((angleX != 0.0) || (angleY != 0.0)) {
              wv.mvMatrix.rotate(angleX, 1,0,0);
              wv.mvMatrix.rotate(angleY, 0,1,0);
              wv.sceneUpd = 1;
            }

        // alt-shift is down (rotate)
        } else if (wv.modifier == 3) {
            var angleX =  (wv.startY - wv.cursorY) / 4.0;
            var angleY = -(wv.startX - wv.cursorX) / 4.0;
            if ((angleX != 0.0) || (angleY != 0.0)) {
              wv.mvMatrix.rotate(angleX, 1,0,0);
              wv.mvMatrix.rotate(angleY, 0,1,0);
              wv.sceneUpd = 1;
            }

        // alt is down (spin)
        } else if (wv.modifier == 2) {
            var xf = wv.startX - wv.width  / 2;
            var yf = wv.startY - wv.height / 2;

            if ((xf != 0.0) || (yf != 0.0)) {
                var theta1 = Math.atan2(yf, xf);
                xf = wv.cursorX - wv.width  / 2;
                yf = wv.cursorY - wv.height / 2;

                if ((xf != 0.0) || (yf != 0.0)) {
                    var dtheta = Math.atan2(yf, xf)-theta1;
                    if (Math.abs(dtheta) < 1.5708) {
                        var angleZ = 128*(dtheta)/3.1415926;
                        wv.mvMatrix.rotate(angleZ, 0,0,1);
                        wv.sceneUpd = 1;
                    }
                }
            }

        // shift is down (zoom)
        } else if (wv.modifier == 1) {
            if (wv.cursorY != wv.startY) {
              var scale = Math.exp((wv.cursorY - wv.startY) / 512.0);
              wv.mvMatrix.scale(scale, scale, scale);
              wv.scale   *= scale;
              wv.sceneUpd = 1;
            }

        // no modifier (translate)
        } else {
            var transX = (wv.cursorX - wv.startX) / 256.0;
            var transY = (wv.cursorY - wv.startY) / 256.0;
            if ((transX != 0.0) || (transY != 0.0)) {
              wv.mvMatrix.translate(transX, transY, 0.0);
              wv.sceneUpd = 1;
            }
        }

        wv.startX = wv.cursorX;
        wv.startY = wv.cursorY;
    }
}





function wvServerMessage(text)
{
	
	var type = "UnknownReturn"
		
	try {
	
		type = text.split(":")[0];
		
	} catch(err) {
		postMessage(err)
		postParseError(text);
	}
	
	
	if (type == "Range") {
		
		try {
			
			wv.keyRange= text.split(":")[1].split(",");
			
			//wv.keyRange[0] = val[0];
			//wv.keyRange[1] = val[1];
			
		} catch(err) {
			postMessage(err)
			postParseError(text);
		}
	} else if (type === "Error") {
		
		var msgAlert = "Something went wrong... please close the browser." + text
		alert(msgAlert)
		
	} else {
		
	 	postMessage(" Server Message: " + text);
	}
	
}


function wvServerDown()
{
    alert("The server has terminated.  You will need to restart the server"
         +" and this browser in order to continue.");
}



// Callback used to put axes on the canvas (called by capsViewer.html)
function wvUpdateCanvas(gl)
{
 // Construct the identity as projection matrix and pass it in
 wv.mvpMatrix.load(wv.perspectiveMatrix);
 wv.mvpMatrix.setUniform(gl, wv.u_modelViewProjMatrixLoc, false);

 var mv   = new J3DIMatrix4();
 var mVal = wv.mvMatrix.getAsArray();

 mVal[ 3] = 0.0;
 mVal[ 7] = 0.0;
 mVal[11] = 0.0;
 mv.load(mVal);
 mv.scale(1.0/wv.scale, 1.0/wv.scale, 1.0/wv.scale);
 mv.invert();
 mv.transpose();

 // define location of axes in space
 var x    = -1.5 * wv.width / wv.height;
 var y    = -1.5;
 var z    =  0.9;
 var alen =  0.25;     // length of axes

 // set up coordinates for axes
 mVal = mv.getAsArray();

 var vertices = new Float32Array(66);
 vertices[ 0] = x;
 vertices[ 1] = y;
 vertices[ 2] = z;
 vertices[ 3] = x + alen*(    mVal[ 0]             );
 vertices[ 4] = y + alen*(    mVal[ 1]             );
 vertices[ 5] = z + alen*(    mVal[ 2]             );
 vertices[ 6] = x + alen*(1.1*mVal[ 0]+0.1*mVal[ 4]);
 vertices[ 7] = y + alen*(1.1*mVal[ 1]+0.1*mVal[ 5]);
 vertices[ 8] = z + alen*(1.1*mVal[ 2]+0.1*mVal[ 6]);
 vertices[ 9] = x + alen*(1.3*mVal[ 0]-0.1*mVal[ 4]);
 vertices[10] = y + alen*(1.3*mVal[ 1]-0.1*mVal[ 5]);
 vertices[11] = z + alen*(1.3*mVal[ 2]-0.1*mVal[ 6]);
 vertices[12] = x + alen*(1.1*mVal[ 0]-0.1*mVal[ 4]);
 vertices[13] = y + alen*(1.1*mVal[ 1]-0.1*mVal[ 5]);
 vertices[14] = z + alen*(1.1*mVal[ 2]-0.1*mVal[ 6]);
 vertices[15] = x + alen*(1.3*mVal[ 0]+0.1*mVal[ 4]);
 vertices[16] = y + alen*(1.3*mVal[ 1]+0.1*mVal[ 5]);
 vertices[17] = z + alen*(1.3*mVal[ 2]+0.1*mVal[ 6]);

 vertices[18] = x;
 vertices[19] = y;
 vertices[20] = z;
 vertices[21] = x + alen*(    mVal[ 4]             );
 vertices[22] = y + alen*(    mVal[ 5]             );
 vertices[23] = z + alen*(    mVal[ 6]             );
 vertices[24] = x + alen*(1.1*mVal[ 4]+0.1*mVal[ 8]);
 vertices[25] = y + alen*(1.1*mVal[ 5]+0.1*mVal[ 9]);
 vertices[26] = z + alen*(1.1*mVal[ 6]+0.1*mVal[10]);
 vertices[27] = x + alen*(1.2*mVal[ 4]             );
 vertices[28] = y + alen*(1.2*mVal[ 5]             );
 vertices[29] = z + alen*(1.2*mVal[ 6]             );
 vertices[30] = x + alen*(1.3*mVal[ 4]+0.1*mVal[ 8]);
 vertices[31] = y + alen*(1.3*mVal[ 5]+0.1*mVal[ 9]);
 vertices[32] = z + alen*(1.3*mVal[ 6]+0.1*mVal[10]);
 vertices[33] = x + alen*(1.2*mVal[ 4]             );
 vertices[34] = y + alen*(1.2*mVal[ 5]             );
 vertices[35] = z + alen*(1.2*mVal[ 6]             );
 vertices[36] = x + alen*(1.2*mVal[ 4]             );
 vertices[37] = y + alen*(1.2*mVal[ 5]             );
 vertices[38] = z + alen*(1.2*mVal[ 6]             );
 vertices[39] = x + alen*(1.2*mVal[ 4]-0.1*mVal[ 8]);
 vertices[40] = y + alen*(1.2*mVal[ 5]-0.1*mVal[ 9]);
 vertices[41] = z + alen*(1.2*mVal[ 6]-0.1*mVal[10]);


 vertices[42] = x;
 vertices[43] = y;
 vertices[44] = z;
 vertices[45] = x + alen*(    mVal[ 8]             );
 vertices[46] = y + alen*(    mVal[ 9]             );
 vertices[47] = z + alen*(    mVal[10]             );
 vertices[48] = x + alen*(1.1*mVal[ 8]+0.1*mVal[ 0]);
 vertices[49] = y + alen*(1.1*mVal[ 9]+0.1*mVal[ 1]);
 vertices[50] = z + alen*(1.1*mVal[10]+0.1*mVal[ 2]);
 vertices[51] = x + alen*(1.3*mVal[ 8]+0.1*mVal[ 0]);
 vertices[52] = y + alen*(1.3*mVal[ 9]+0.1*mVal[ 1]);
 vertices[53] = z + alen*(1.3*mVal[10]+0.1*mVal[ 2]);
 vertices[54] = x + alen*(1.3*mVal[ 8]+0.1*mVal[ 0]);
 vertices[55] = y + alen*(1.3*mVal[ 9]+0.1*mVal[ 1]);
 vertices[56] = z + alen*(1.3*mVal[10]+0.1*mVal[ 2]);
 vertices[57] = x + alen*(1.1*mVal[ 8]-0.1*mVal[ 0]);
 vertices[58] = y + alen*(1.1*mVal[ 9]-0.1*mVal[ 1]);
 vertices[59] = z + alen*(1.1*mVal[10]-0.1*mVal[ 2]);
 vertices[60] = x + alen*(1.1*mVal[ 8]-0.1*mVal[ 0]);
 vertices[61] = y + alen*(1.1*mVal[ 9]-0.1*mVal[ 1]);
 vertices[62] = z + alen*(1.1*mVal[10]-0.1*mVal[ 2]);
 vertices[63] = x + alen*(1.3*mVal[ 8]-0.1*mVal[ 0]);
 vertices[64] = y + alen*(1.3*mVal[ 9]-0.1*mVal[ 1]);
 vertices[65] = z + alen*(1.3*mVal[10]-0.1*mVal[ 2]);

 // set up colors for the axes
 var colors = new Uint8Array(66);
 colors[ 0] = 255;   colors[ 1] =   0;   colors[ 2] =   0;
 colors[ 3] = 255;   colors[ 4] =   0;   colors[ 5] =   0;
 colors[ 6] = 255;   colors[ 7] =   0;   colors[ 8] =   0;
 colors[ 9] = 255;   colors[10] =   0;   colors[11] =   0;
 colors[12] = 255;   colors[13] =   0;   colors[14] =   0;
 colors[15] = 255;   colors[16] =   0;   colors[17] =   0;
 colors[18] =   0;   colors[19] = 255;   colors[20] =   0;
 colors[21] =   0;   colors[22] = 255;   colors[23] =   0;
 colors[24] =   0;   colors[25] = 255;   colors[26] =   0;
 colors[27] =   0;   colors[28] = 255;   colors[29] =   0;
 colors[30] =   0;   colors[31] = 255;   colors[32] =   0;
 colors[33] =   0;   colors[34] = 255;   colors[35] =   0;
 colors[36] =   0;   colors[37] = 255;   colors[38] =   0;
 colors[39] =   0;   colors[40] = 255;   colors[41] =   0;
 colors[42] =   0;   colors[43] =   0;   colors[44] = 255;
 colors[45] =   0;   colors[46] =   0;   colors[47] = 255;
 colors[48] =   0;   colors[49] =   0;   colors[50] = 255;
 colors[51] =   0;   colors[52] =   0;   colors[53] = 255;
 colors[54] =   0;   colors[55] =   0;   colors[56] = 255;
 colors[57] =   0;   colors[58] =   0;   colors[59] = 255;
 colors[60] =   0;   colors[61] =   0;   colors[62] = 255;
 colors[63] =   0;   colors[64] =   0;   colors[65] = 255;

 // draw the axes
 if (gl.lineWidth) {
     gl.lineWidth(2);
 }
 gl.disableVertexAttribArray(2);
 gl.uniform1f(wv.u_wLightLoc, 0.0);

 var buffer = gl.createBuffer();
 gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
 gl.bufferData(gl.ARRAY_BUFFER, vertices, gl.STATIC_DRAW);
 gl.vertexAttribPointer(0, 3, gl.FLOAT, false, 0, 0);
 gl.enableVertexAttribArray(0);

 var cbuf = gl.createBuffer();
 gl.bindBuffer(gl.ARRAY_BUFFER, cbuf);
 gl.bufferData(gl.ARRAY_BUFFER, colors, gl.STATIC_DRAW);
 gl.vertexAttribPointer(1, 3, gl.UNSIGNED_BYTE, false, 0, 0);
 gl.enableVertexAttribArray(1);

 gl.drawArrays(gl.LINES, 0, 22);
 gl.deleteBuffer(buffer);
 gl.deleteBuffer(cbuf);
 gl.uniform1f(wv.u_wLightLoc, 1.0);
}


//function turnOnColor() {
//	
//	for (var gprim in wv.sceneGraph) { // Loop through scene primatives
			
		//wv.sceneGraph[gprim].attrs |=  wv.plotAttrs.SHADING;
		//wv.sceneGraph[gprim].attrs |=  wv.plotAttrs.LINES;
//	}	
//}

// Callback when the mouse is pressed in key window
function setKeyLimits(e) {
	
	try {
		wv.lowerLimit = wv.key.bottom;
		wv.upperLimit = wv.key.top;
	} catch(err) {
		postMessage("Error: no key set yet!")
		return 
	} 
	
	var msg = "Enter upper limit (max = " + wv.keyRange[1] + ")" 
	var tempup = prompt(msg, wv.upperLimit);
	if (tempup == null) {
		return; 
	} 
	
	if (isNaN(tempup)) {
		alert("Upper limit must be a number");
		return;
	}
	
	// Get new limits
	msg = "Enter lower limit (min = " + wv.keyRange[0] + ")" 
	var templo = prompt(msg, wv.lowerLimit);
	
	if (templo == null) {
		return; 
	} 
	
	if (isNaN(templo)) {
     alert("Lower limit must be a number");
     return;
	}


	if (Number(tempup) < Number(templo)) {
		alert("Upper limit must be greater than lower limit");
		return;
	}

	if (templo != wv.lowerLimit || tempup != wv.uplimit) {
		wv.lowerLimit = templo;
		wv.upperLimit = tempup;

		// send the limits back to the server
		var limits = "limits,"+ wv.lowerLimit.toString() + "," + wv.upperLimit.toString();
		browserToServer(limits);
	}
	
	wv.drawKey = 1;
}