# C-Imports 
cimport cWV
cimport cCAPS

import webbrowser
import time 
from struct import unpack as structUnpack

#cdef int numInstance

ctypedef struct globalContainer:
    void ** gprim
    int numPrim

cdef globalContainer globalInstance

#cdef cWV.wvContext *globalInstanceontext

cdef clearGlobal():
    global globalInstance
    
    if globalInstance.gprim:
        free(globalInstance.gprim)
        
    globalInstance.gprim = NULL 
    
    globalInstance.numPrim = 0
    
    
# Set the scalar color map associated with data. It is assumed data is in a list. 
# Return a void * (must be freed) for the scalar color map (which if of float type) 
cdef void * setScalarColor(object data, double dataMin, double dataMax, int numContour, int reverseMap, char * cMapName):

    cdef: 
        float *color = NULL
        double scalar, frac
        int index, numColor
        
    numData = len(data)

    color = <float *> malloc(3*numData*sizeof(float))
    if not color:
        raise MemoryError()
    
    colorMap = getColorMap(<object> cMapName, numContour, reverseMap)
    numColor = len(colorMap)/3 - 1
    
    if numColor == 0:
        dataMin = dataMax

    i = 0
    for scalar in data:
        if (dataMin == dataMax):
            color[3*i + 0] = <float> colorMap[-3]
            color[3*i + 1] = <float> colorMap[-2]
            color[3*i + 2] = <float> colorMap[-1]
  
        elif scalar <= dataMin:
            color[3*i + 0] = <float> colorMap[0]
            color[3*i + 1] = <float> colorMap[1]
            color[3*i + 2] = <float> colorMap[2]
  
        elif scalar >= dataMax:
            color[3*i + 0] = <float> colorMap[-3]
            color[3*i + 1] = <float> colorMap[-2]
            color[3*i + 2] = <float> colorMap[-1]
            
        else:
    
            frac  = numColor * (scalar- dataMin) / (dataMax- dataMin)
            
            if (frac < 0):
                frac = 0
            if (frac > numColor):
                frac = numColor
    
            index  = int(frac)
            
            #frac -= index
            
            #if index == numColor:
            #    index = index -1
            #   frac += 1.0
    
            #color[3*i + 0] = <float > frac * colorMap[3*(index+1)+0] + (1.0-frac) * colorMap[3*index+0]
            #color[3*i + 1] = <float > frac * colorMap[3*(index+1)+1] + (1.0-frac) * colorMap[3*index+1]
            #color[3*i + 2] = <float > frac * colorMap[3*(index+1)+2] + (1.0-frac) * colorMap[3*index+2]
            
            color[3*i + 0] = <float> colorMap[3*(index+1)+0]
            color[3*i + 1] = <float> colorMap[3*(index+1)+1]
            color[3*i + 2] = <float> colorMap[3*(index+1)+2]
        
        #print color[3*i + 0], color[3*i + 1], color[3*i + 2]
        i = i + 1
        
    return <void *> color

# Required call-back to catch browser messages
cdef public void browserMessage(void *wsi, char *inText, int lenText):
    
    cdef:
        void * temp
        char * msg
        char *limits
        double upper, lower
        
        int i=0, istart=0, iend=0
        
        double *colorRange 
        
        int numContour
        
        char msgFinite[100]
    
    msg = msgFinite
    
    if b'nextScalar' == inText:
        
        for i in range(globalInstance.numPrim):
            temp = globalInstance.gprim[i]
        
            prim = <graphicPrimitive > temp
            
            prim.raiseException = False
            
            msg = prim.nextColor()
            
            if (msg == NULL):
                msg = "No colors!"
            
            prim.raiseException = True
            
            if prim.status < 0:
                msg = msgFinite
                sprintf(msg, "Error:%+d", prim.status)
                break
                 
        cWV.wv_sendText(wsi, msg)
    
    elif b'previousScalar' == inText:
        
        for i in range(globalInstance.numPrim):
            temp = globalInstance.gprim[i]
        
            prim = <graphicPrimitive > temp
            
            prim.raiseException = False
            
            msg = prim.previousColor()
            if (msg == NULL):
                msg = "No colors!"
            
            prim.raiseException = True
            
            if prim.status < 0:
                msg = msgFinite
                sprintf(msg, "Error:%+d", prim.status)
                break
                 
        cWV.wv_sendText(wsi, msg)
            
    elif b"limits" in inText:
        
        for i in range(lenText):
                
            if ',' == inText[i] and istart == 0:
                istart = i
            elif ',' == inText[i]:
                iend = i
        
        # Need special treatment for negatives to a avoid a seg-fault in PyFloat_FromString
        if inText[istart+1] == b"-": 
            lower = <double > float(inText[istart+2:iend])
            lower = -lower
        else: 
            lower = <double > float(inText[istart+1:iend])
        
        if inText[iend+1] == b"-":
            upper = <double > float(inText[iend+2:])
            upper = -upper 
        else:
            upper = <double > float(inText[iend+1:])

        for i in range(globalInstance.numPrim):
            temp = globalInstance.gprim[i]
        
            prim = <graphicPrimitive > temp
            
            prim.raiseException = False
              
            prim.changeLimit(lower, upper)
            
            prim.raiseException = True
            
            
            msg = "Limits updated"
            if prim.status < 0:
                msg = msgFinite
                sprintf(msg, "Error:%+d", prim.status)
                break
        
        cWV.wv_sendText(wsi, msg)
        
    elif b"colorMap," in inText:
        
        for i in range(globalInstance.numPrim):
            temp = globalInstance.gprim[i]
        
            prim = <graphicPrimitive > temp
            
            prim.raiseException = False
              
            prim.changeColorMap(inText[9:])
            
            prim.raiseException = True
            
            msg = "ColorMap updated"
            if prim.status < 0:
                msg = msgFinite
                sprintf(msg, "Error:%+d", prim.status)
                break
        
        cWV.wv_sendText(wsi, msg)
        
    elif b"reverseMap" in inText:
        
        for i in range(globalInstance.numPrim):
            temp = globalInstance.gprim[i]
        
            prim = <graphicPrimitive > temp
            
            prim.raiseException = False
              
            prim.reverseColorMap()
            
            prim.raiseException = True
            
            msg = "ColorMap reversed/inverted"
            if prim.status < 0:
                msg = msgFinite
                sprintf(msg, "Error:%+d", prim.status)
                break
        
        cWV.wv_sendText(wsi, msg)
        
    elif b"getRange" in inText:
    
        for i in range(globalInstance.numPrim):
            temp = globalInstance.gprim[i]
        
            prim = <graphicPrimitive > temp
            
            prim.raiseException = False
            
            colorRange = prim.getRange()
            
            sprintf(msgFinite, "Range:%10.6g,%10.6g", colorRange[0], colorRange[1])
            
            prim.raiseException = True
            
            if prim.status < 0:
                msg = msgFinite
                sprintf(msg, "Error:%+d", prim.status)
                break
        
        cWV.wv_sendText(wsi, msg)
        
    elif b"contourLevel," in inText:
        
        istart = 12
        
        # Need special treatment for negatives to a avoid a seg-fault in PyFloat_FromString
        if inText[istart+1] == "-": 
            numContour = < int > float(inText[istart+2:])
            numContour = -numContour
        else: 
            numContour = < int > float(inText[istart+1:])
            
        for i in range(globalInstance.numPrim):
            temp = globalInstance.gprim[i]
        
            prim = <graphicPrimitive > temp
            
            prim.raiseException = False
              
            prim.changeContourLevel(numContour)
            
            prim.raiseException = True
            
            msg = "Contour level updated"
            if prim.status < 0:
                msg = msgFinite
                sprintf(msg, "Error:%+d", prim.status)
                break
        
        cWV.wv_sendText(wsi, msg)
                
    else:
        cWV.wv_sendText(wsi, "Command not recognized!")
        
    return 

# Copy an item. Note that isn't a deep copy - the data in the data pointer isn't being copied 
cdef void copyItem(cWV.wvData *inVal, cWV.wvData *outVal):
    
    outVal.dataType = inVal.dataType   #     /* VBO type */
    outVal.dataLen  = inVal.dataLen    #    /* length of data */
    outVal.dataPtr  = inVal.dataPtr    #     /* pointer to data */

    outVal.data[0] = inVal.data[0] #    /* size = 1 data (not in ptr) *
    outVal.data[1] = inVal.data[1] #    /* size = 1 data (not in ptr) *
    outVal.data[2] = inVal.data[2] #    /* size = 1 data (not in ptr) *
        
cdef class wvData:
    cdef:
        int status
        
        cWV.wvData item
        cWV.wvData *itemPointer
        
        public object name # Name of the item. Keep as an object to avoid string memory complications
        
        public list colorData # Need to store way the color data in case we are going to change the limits
        public object colorMap # Name of the color map (if any). Keep as an object to avoid string memory complications
        
        public double minColor 
        public double maxColor 
        
        public double minColorRange  # Initial max and min color value 
        public double maxColorRange   
        
        public int numContour
        public int reverseMap
    
        public int raiseException
        
        #public float focus[4]
        public float boundingBox[6] # Xmin, Xmax, Ymin, Ymax, Zmin, Zmax
        
    def __cinit__(self):
        
        cdef: 
            int i
            char *nameTemp = "UnknownVariable"
            
        self.status = 0
        
        self.itemPointer = NULL
        
        self.name = nameTemp
               
        self.colorData = []
        self.colorMap = None
        
        self.minColor = -1.0
        self.maxColor = 1.0
        
        self.minColorRange = cCAPS.CAPSMAGIC
        self.maxColorRange = cCAPS.CAPSMAGIC
        
        self.numContour = 0
        
        self.reverseMap = <int> False
        
        self.raiseException = True
        
        #self.focus[0] = 0.0
        #self.focus[1] = 0.0
        #elf.focus[2] = 0.0
        #self.focus[3] = 0.0
        
        for i in range(6):
            self.boundingBox[i] = 0.0
        
    def __del__(self):
        cWV.wv_freeItem(&self.item)
        
        self.itemPointer = NULL
        
    def __dealloc__(self):
        cWV.wv_freeItem(&self.item)
        
        self.itemPointer = NULL
        
    @property
    def __wvDataItem(self):
        
        return <object> self.itemPointer
    
    cdef freeItem(self):
        cWV.wv_freeItem(&self.item)
        
    cdef createVertex(self, object xyz, char *name = NULL):
        
        if name:
            self.name = name
            
        if not isinstance(xyz, list):
            xyz = [xyz]
        
        length = len(xyz)
        
        data = castValue2VoidP(xyz, None, cCAPS.Double)
                            
        self.status = cWV.wv_setData(cWV.WV_REAL64, <int> length, data,  cWV.WV_VERTICES, &self.item)
        if data:
            free(data)
        
        self.itemPointer = &self.item
        
        if self.raiseException:
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while setting data (createVertex), code error "+str(self.status))
        
        return self 
    
    cdef createIndex(self, object connectivity, char *name = NULL):
        
        cdef: 
            int length=0
            object removeList = []

        if name:
            self.name = name
            
        connect = connectivity[:] # Create a copy so we don't change the initial object
            
        for i, val in enumerate(connectivity):
            
            if len(val) > 3:
                
                removeList.append(i)
                
                for j in range(len(val)-2): # 4 goes to 2; 5 -> 3, 6 -> 4 
                    
                    connect.append([val[0], val[j+1], val[j+2]])
                    
                    length += len(connect[-1])
            else:
                length += len(val)
        
        for i in removeList:
            del connect[i]
        
        data = castValue2VoidP(connect, None, cCAPS.Integer)
        
        self.status = cWV.wv_setData(cWV.WV_INT32, length, data, cWV.WV_INDICES, &self.item)
        if data:
            free(data)
            
        self.itemPointer = &self.item
        
        if self.raiseException:
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while setting data (createIndex), code error "+str(self.status))
        
        return self
    
    cdef createLineIndex(self, object connectivity, char *name = NULL):
        
        cdef: 
            int length=0 

        if name:
            self.name = name
            
        segment = []
        for i in connectivity:
            
            for j in range(len(i)):
                
                segment.append(i[j])
                length += 1
                
                if j == len(i)-1:
                    segment.append(i[0])
                
                else:
                    segment.append(i[j+1])
                
                length += 1
                
        data = castValue2VoidP(segment, None, cCAPS.Integer)

        self.status = cWV.wv_setData(cWV.WV_INT32, length, data, cWV.WV_LINDICES, &self.item);
        if data:
            free(data)
    
        self.itemPointer = &self.item
        
        if self.raiseException:
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while setting data (createIndex), code error "+str(self.status))
        
        return self
        
    cpdef updateColor(self, double minColor, double  maxColor, int numContour, int reverseMap, char *colorMap):
    
        cdef: 
            int length
            void *data = NULL
        
        self.freeItem()
            
        length = len(self.colorData)
        
        self.minColor = minColor 
        self.maxColor = maxColor
        self.numContour = numContour
        self.reverseMap = reverseMap
        
        if colorMap:
            self.colorMap = colorMap    
        
        # Set color returns a void * to a float *
        data = setScalarColor(self.colorData, 
                              self.minColor, self.maxColor, 
                              self.numContour, self.reverseMap, 
                              <char *> self.colorMap)

        self.status = cWV.wv_setData(cWV.WV_REAL32, length, data, cWV.WV_COLORS, &self.item)
        if data:
            free(data)
        
        self.itemPointer = &self.item
        
        if self.raiseException:
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while setting data (updateColor), code error "+str(self.status))
        
        return self
    
    cdef createColor(self, object colorData, double minColor, double  maxColor, int numContour, int reverseMap, char * colorMap, char * name=NULL): 
        cdef: 
            int length
            void *data = NULL
        
        self.freeItem()
        
        if colorMap:
            self.colorMap = colorMap 
                 
        if self.minColorRange == cCAPS.CAPSMAGIC:
            self.minColorRange = minColor
        
        if self.maxColorRange == cCAPS.CAPSMAGIC:
            self.maxColorRange = maxColor 
        
        self.minColor = minColor 
        self.maxColor = maxColor
        
        self.numContour = numContour
        
        self.reverseMap = reverseMap
        
        if name:
            self.name = name
        
        if not isinstance(colorData, list):
            colorData = [colorData]
        
        self.colorData = colorData
                
        length = len(colorData)
        
        # Set color returns a void * to a float *
        data = setScalarColor(colorData, self.minColor, self.maxColor, self.numContour, self.reverseMap, self.colorMap)

        self.status = cWV.wv_setData(cWV.WV_REAL32, length, data, cWV.WV_COLORS, &self.item)
        if data:
            free(data)
        
        self.itemPointer = &self.item
        
        if self.raiseException:
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while setting data (createColor), code error "+str(self.status))
        
        return self
    
    cdef createLineColor(self, object colorData, double minColor, double maxColor, int numContour, int reverseMap, char *colorMap=NULL, char *name = NULL):
        
        cdef: 
            int length
            void *data = NULL
        
        if colorMap:
            self.colorMap = colorMap
            
        if name:
            self.name = name
            
        if not isinstance(colorData, list):
            colorData = [colorData]
            
        length = len(colorData)
        
        # Set color returns a void * to a float *
        data = setScalarColor(colorData, minColor, maxColor, numContour, reverseMap, colorMap)

        self.status = cWV.wv_setData(cWV.WV_REAL32, length, data, cWV.WV_LCOLOR, &self.item)
        if data:
            free(data)
        
        self.itemPointer = &self.item
        
        if self.raiseException:
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while setting data (createLineColor), code error "+str(self.status))
        
        return self
    
    def setBoundingBox(self, object box):
        if len(box) < 6:
            self.status = cCAPS.CAPS_RANGEERR
            
            if self.raiseException:
                raise CAPSError(self.status, msg="while setting bounding box data")
        
        for i in range(6):
            self.boundingBox[i] = box[i]
            
    def adjustVertex(self, object focus):
        cdef: 
            float focal[4]
            
        if len(focus) < 4:
            self.status = cCAPS.CAPS_RANGEERR
            
            if self.raiseException:
                raise CAPSError(self.status, msg="while setting adjust vertex data")
            else:
                return 
        
        for i in range(4):
            focal[i] = focus[i]   
            
        cWV.wv_adjustVerts(&self.item, focal)


cdef class graphicPrimitive:
    cdef:
        capsViewer viewer # Viewer class
        
        int status 
        
        readonly object colors # List of color wvData object 
        
        int currentColor
        
        int primIndex
        
        int raiseException
        
        readonly object name # Name of the primitive
        
        double range[2]
        
        readonly object vertex # Copy of vertices wvData object
        
    def __cinit__(self, viewer):
        
        cdef: 
            char *tempName = "UnknownPrimitive"
            
        if not isinstance(viewer, capsViewer):
            raise TypeError
        
        self.viewer = viewer
        
        self.colors = []
        self.currentColor = 0
        
        self.status = 0
        
        self.primIndex = 0
        
        self.raiseException = True
        
        self.name = tempName
        
        self.range[0] = -1.0
        self.range[1] = 1.0
        
        self.vertex = None
        
    def __del__(self):
        
        self.primIndex = 0
        
        self.colors =[]
        self.currentColor = 0
        
        self.viewer = None
        
    def __dealloc__(self):
        
        self.primIndex = 0
        
        self.colors =[]
        self.currentColor = 0 
        
        self.viewer = None
 
    # Append a or multiple colors to the color list after even after a primitive has been created 
    cpdef appendColor(self, object items):
    
        if not isinstance(items, list):
            items = [items]
            
        if not isinstance(items[0], wvData):
            if self.raiseException: 
                raise TypeError
            else:
                self.status = -99
                return self.status
            
        for i in items:
            temp = <cWV.wvData *> i.__wvDataItem
           
            if temp.dataType == cWV.WV_COLORS:
                self.colors.append(i)
                
    # Update vertex - assumes the item the "vertex" corresponds to has been changed (i.e. a adjustVertex call was made)
    cpdef updateVertex(self):
        
        if self.colors:
            
            self.modifyPrimitive([self.vertex, self.colors[self.currentColor-1]])
        else : 
            
            self.modifyPrimitive(self.vertex)
    
    # \param items List of wvData objects used to create the triangle 
    # \param name Name of graphic primitive. Important: Name must be unique 
    # \return Instance of  graphicPrimitive class 
    def createPoint(self, items, name):
        
        if not isinstance(items, list):
            items = [items]
            
        if not isinstance(items[0], wvData):
            raise TypeError
            
        return self.createPrimative(items, name, "point")
    
    # \param items List of wvData objects used to create the triangle 
    # \param name Name of graphic primitive. Important: Name must be unique 
    # \return Instance of  graphicPrimitive class 
    def createLine(self, items, name):
        
        if not isinstance(items, list):
            items = [items]
            
        if not isinstance(items[0], wvData):
            raise TypeError
            
        return self.createPrimative(items, name, "line")
        
    # \param items List of wvData objects used to create the triangle 
    # \param name Name of graphic primitive. Important: Name must be unique 
    # \return Instance of  graphicPrimitive class 
    def createTriangle(self, items, name):
        
        if not isinstance(items, list):
            items = [items]
            
        if not isinstance(items[0], wvData):
            raise TypeError
            
        return self.createPrimative(items, name, "triangle")
    
    # \return The item name for the color being set 
    cdef char *nextColor(self):
        
        cdef:
            object item 
            
        if len(self.colors) == 0:
            return <char *> NULL 
        
        if self.currentColor == len(self.colors):
        
            self.currentColor = 1
        
        else:
            self.currentColor += 1 # if not the last color
            
        item = self.colors[self.currentColor -1]
        
        # Update primitive with the new color 
        self.modifyPrimitive(item)
        
        if not self.raiseException:
            if self.status < 0:
                return NULL
            
        # Update the key
        self.setKey()
        
        if not self.raiseException:
            if self.status < 0:
                return NULL
        
        # Return the item's name 
        return <char *> item.name
    
    # \return The item name for the color being set 
    cdef char * previousColor(self):
        
        cdef:
            object item 
        
        if len(self.colors) == 0:
            return <char *> NULL
        
        if self.currentColor == 1:
        
            self.currentColor = len(self.colors)
        
        else:
          
            self.currentColor -= 1 # if not the first color 
            
        item = self.colors[self.currentColor -1]
        
        # Update primitive with the new color 
        self.modifyPrimitive(item)
        
        if not self.raiseException:
            if self.status < 0:
                return NULL
        
        # Update the key
        self.setKey()
        
        if not self.raiseException:
            if self.status < 0:
                return NULL
    
        # Return the item's name 
        return <char *> item.name
    
    # Change the limits of the current data set 
    cdef void changeLimit(self, double minColor, double maxColor):
        cdef:
            wvData item
        
        if len(self.colors) == 0:
            return
        
        # Create a new color based on the current colors data, but change the limits
        item = self.colors[self.currentColor -1]
       
        item.raiseException = False
        
        item.updateColor(minColor, maxColor, item.numContour, item.reverseMap, item.colorMap)
         
        if not item.raiseException:
            if item.status < 0:
                
                item.raiseException = True
               
                self.status = item.status
                
                return 
        
        item.raiseException = True
        
        # Update primitive with the new color 
        self.modifyPrimitive(item)
        
        if not self.raiseException:
            if self.status < 0:
                return 
         
        # Update the key
        self.setKey()
        
        if not self.raiseException:
            if self.status < 0:
                return 
    
    # Change all the color maps 
    cdef void changeColorMap(self, char *colorMap):
        cdef:
            wvData item
            int i 
        
        if len(self.colors) == 0:
            return
        
        for i in range(len(self.colors)):
             
            # Create a new color based on the current colors data, but change the colorMap
            item = self.colors[i]
            
            item.raiseException = False
            
            item.updateColor(item.minColor, item.maxColor, item.numContour, item.reverseMap, colorMap)
            
            if not item.raiseException:
                if item.status < 0:
                
                    item.raiseException = True
               
                    self.status = item.status
                
                    return 
        
            item.raiseException = True
            
            # Only update the primitive if it is our current color 
            if i == self.currentColor-1:
                self.modifyPrimitive(item)
                
                if not self.raiseException:
                    if self.status < 0:
                        return 
                
        # Update the key
        self.setKey()
        
        if not self.raiseException:
            if self.status < 0:
                return 
    
    # Change all the contour levels  
    cdef void changeContourLevel(self, int numContour):
        cdef:
            wvData item
            int i 
        
        if len(self.colors) == 0:
            return
        
        for i in range(len(self.colors)):
             
            # Create a new color based on the current colors data, but change the colorMap
            item = self.colors[i]
            
            item.raiseException = False
            
            item.updateColor(item.minColor, item.maxColor, numContour, item.reverseMap, item.colorMap)
            
            if not item.raiseException:
                if item.status < 0:
                
                    item.raiseException = True
               
                    self.status = item.status
                
                    return 
        
            item.raiseException = True
            
            # Only update the primitive if it is our current color 
            if i == self.currentColor-1:
                self.modifyPrimitive(item)
                
                if not self.raiseException:
                    if self.status < 0:
                        return 
    
        # Update the key
        self.setKey()
        
        if not self.raiseException:
            if self.status < 0:
                return 
    
    # Reverse/invert all the color maps  
    cdef void reverseColorMap(self):
        cdef:
            wvData item
            int i 
        
        if len(self.colors) == 0:
            return
        
        for i in range(len(self.colors)):
             
            # Create a new color based on the current colors data, but change the colorMap
            item = self.colors[i]
            
            item.raiseException = False
            
            if item.reverseMap == True:
                item.updateColor(item.minColor, item.maxColor, item.numContour, False, item.colorMap)
            else:
                item.updateColor(item.minColor, item.maxColor, item.numContour, True, item.colorMap)
            
            if not item.raiseException:
                if item.status < 0:
                
                    item.raiseException = True
               
                    self.status = item.status
                
                    return 
        
            item.raiseException = True
            
            # Only update the primitive if it is our current color 
            if i == self.currentColor-1:
                self.modifyPrimitive(item)
                
                if not self.raiseException:
                    if self.status < 0:
                        return 
    
        # Update the key
        self.setKey()
        
        if not self.raiseException:
            if self.status < 0:
                return 
    # Return the max and min initial color values (range)
    cdef double *getRange(self):
        cdef:
            double * range = NULL
        
        if len(self.colors) == 0:
            return self.range
        
        # Create a new color based on the current colors data, but change the limits
        item = self.colors[self.currentColor -1]
        
        self.range[0] = item.minColorRange
        self.range[1] = item.maxColorRange
        
        return self.range
    
    # Set the key for the context
    cdef void setKey(self):
        
        cdef:
            wvData item
            
            float *keyData = NULL
            int i, numCol
            
            char *title
            
            object cMapName
            
            float minColor 
            float maxColor 
            
            object colorMap
            
            int numContour 
            int reverseMap
            
        if len(self.colors) == 0:
            return
        
        # Get current color and it's information 
        item = self.colors[self.currentColor -1]
        
        
        title    = <char*>  item.name
        minColor = <float>  item.minColor
        maxColor = <float>  item.maxColor
        cMapName = <object> item.colorMap
        numContour = <int>  item.numContour
        reverseMap = <int>  item.reverseMap
        
        # Get color map 
        colorMap = getColorMap(cMapName, numContour, reverseMap)
        
        # Create key data 
        if minColor == maxColor:
        
            numCol = 1
        
        else:
            
            numCol = <int> len(colorMap)/3
        
        keyData = <float *> malloc(3*numCol*sizeof(float))
        if not keyData:
            raise MemoryError
        
        if minColor == maxColor:
            keyData[0] = <float> colorMap[-3]
            keyData[1] = <float> colorMap[-2]
            keyData[2] = <float> colorMap[-1]
        else:
            
            for i in range(3*numCol):
                keyData[i] = <float> colorMap[i]
            
        self.status = cWV.wv_setKey(self.viewer.context, numCol,  
                                                         keyData, 
                                                         minColor, maxColor, 
                                                         title)
        
        if keyData:
           free(keyData)
        
        if self.raiseException:
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while setting key (setKey), code error "+str(self.status))
    
    cdef int createPrimative(self, list items, char * name, char *type) except -1:
        cdef:
            
            int numItem = 0
            cWV.wvData *tempItem =NULL
            
            cWV.wvData *temp
            
            #int colorCount = 0
        
        # Store away the name
        self.name = name
                
        for i in items:
            
            temp = <cWV.wvData *> i.__wvDataItem
            
            if temp.dataType == cWV.WV_COLORS:
                self.appendColor(i)
            
                if len(self.colors) > 1:
                    continue
                elif len(self.colors) == 1:
                    self.currentColor = 1
            
            #   self.currentColor = 1
            #   self.colors.append(i) # Store the wvData instance 
                
            #    colorCount += 1
                
            #if  temp.dataType == cWV.WV_COLORS and colorCount > 1: 
            # 
            #     continue
            
            if not self.vertex:
                if temp.dataType == cWV.WV_VERTICES:
                    self.vertex = i
        
            # Copy the item data temporarily 
            numItem += 1
            
            tempItem = <cWV.wvData *> realloc(tempItem, <int>numItem*sizeof(cWV.wvData))
            if not tempItem:
                if self.raiseException: 
                    raise MemoryError
                else:
                    self.status = -99
                    return self.status 
                
            # Note that isn't a deep copy - the data in the data pointer isn't being copied 
            copyItem(temp, &tempItem[numItem-1])
        
        if _strify(type) == "point":
            self.status = cWV.wv_addGPrim(self.viewer.context,
                                          <char *> name, 
                                          cWV.WV_POINT, 
                                          cWV.WV_ON+cWV.WV_ORIENTATION,
                                          numItem, 
                                          tempItem)
        elif _strify(type) == "line":
            self.status = cWV.wv_addGPrim(self.viewer.context,
                                          <char *> name, 
                                          cWV.WV_LINE, 
                                          cWV.WV_ON+cWV.WV_ORIENTATION+cWV.WV_LINES,
                                          numItem, 
                                          tempItem)
        elif _strify(type) == "triangle":
            self.status = cWV.wv_addGPrim(self.viewer.context,
                                          <char *> name, 
                                          cWV.WV_TRIANGLE, 
                                          cWV.WV_ON+cWV.WV_ORIENTATION+cWV.WV_LINES+cWV.WV_SHADING,
                                          numItem, 
                                          tempItem)
        else: 
            
            if tempItem:
                free(tempItem)
            
            tempItem = NULL
            
            if self.raiseException: 
                raise TypeError
            else:
                self.status = -99
                return self.status 
                 
        if tempItem:
                free(tempItem)
        
        if self.raiseException:      
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while adding graphic primitive (wv_addGPrim), code error "+str(self.status))

        self.primIndex = self.status
        
        return <int> self.primIndex
              
    cdef void modifyPrimitive(self, items):
        
        cdef: 
           cWV.wvData *tempItem =NULL
           cWV.wvData *temp
           
           
        if not isinstance(items, list):
            items = [items]
            
        if not isinstance(items[0], wvData):
            raise TypeError
        
        tempItem = <cWV.wvData *> malloc(len(items)*sizeof(cWV.wvData))
        if not tempItem:
            raise MemoryError
        
        for i, item in enumerate(items):
            
            temp = <cWV.wvData *> item.__wvDataItem
            
            # Note that isn't a deep copy - the data in the data pointer isn't being copied 
            copyItem(temp, &tempItem[i])
            
        self.status = cWV.wv_modGPrim(self.viewer.context, 
                                      self.primIndex,
                                      <int> len(items), 
                                      tempItem)
        if tempItem:
            free(tempItem)
        
        if self.raiseException:
            
            if self.status < 0:
                raise CAPSError(self.status-1000, msg="while modifying graphic primitive (wv_modGPrim), code error"+str(self.status))
        
## Defines a CAPS viewer object.
# A capsViewer object is a Pythonized version of Bob Haimes's "wv: A General Web-based 3D Viewer" API. wv's goal is "to
# generate a visual tool targeted for the 3D needs found within the MDAO process. A WebBrowser-based approach 
# is considered, in that it provides the broadest possible platform for deployment."
cdef class capsViewer:
    ## \example webviewer.py 
    # Basic example use case for creating visualizing data using CAPS's web viewer \ref pyCAPS.capsViewer
    
    cdef:
     
        int status 
        
        cWV.wvContext *context
       
        object controller
        object portNumber
        
        int serverInstance
        
        object viewerHTML
        
        int numGPrim
        object graphicPrim 
        object graphicPrimName
        
        # List of items that have been added 
        object items # Must store all items
        
        object currentItems
        
        int keyNumCol
        float *keyData
        float keyMax
        float keyMin
        object keyTitle
        
        object oneBias
        
    ## Initialize the viewer object. 
    # See \ref webviewer.py for an example use case.
    # \param browserName Name of browser to load (default - None). If left as None the system level default browser will 
    # be used. 
    #
    # \param portNumber Port number to start the server listening on (default - 7681).
    #
    # \param oneBias Flag to indicate the index biasing of the data. 
    # Options: 1 bias (default - True) or 0 bias (False) indexed.
    #
    # \param html Specify an alternative HTML file to launch when starting the server, 
    # see \ref startServer (default - "file:/$ESP_ROOT/lib/capsViewer.html")
    def __cinit__(self, browserName = None, portNumber=7681, oneBias=True, 
                  html="file:/$ESP_ROOT/lib/capsViewer.html"):
        
        cdef:
            float eye[3]
            float center[3]
            float up[3]
            
        self.portNumber = portNumber
        
        if html == "file:/$ESP_ROOT/lib/capsViewer.html":
            try:
                html = os.path.join("file:", os.environ["ESP_ROOT"])
                html = os.path.join(html, "lib")
                html = os.path.join(html, "capsViewer.html")
            except:
                
                print "Unable to determine $ESP_ROOT environment variable. Browser will NOT start automatically!"
                html = None
                
        self.viewerHTML = html
        
        if self.viewerHTML:
            
            if not os.path.isfile(self.viewerHTML.replace("file:", "")):
            
                print "Unable to locate file -", self.viewerHTML, ". Browser will NOT start automatically!"
                self.viewerHTML = None
                
        self.context = NULL
    
        self.items = []
        self.currentItems = []
        
        self.graphicPrim = []
        self.graphicPrimName = []
        
        self.oneBias = oneBias 

        eye[:] = [0.0, 0.0, 7.0]
        center[:] = [0.0, 0.0, 0.0]
        up[:] = [0.0, 1.0, 0.0]
        
        self.controller = None 
        
        if self.viewerHTML:
            
            self.controller = webbrowser.get(browserName)
            
            if not self.controller:
                self.status = cCAPS.CAPS_NOTFOUND
                raise CAPSError(self.status, msg="while trying to find browser - " + browserName)
            
        self.context = cWV.wv_createContext(<int> int(self.oneBias), 30.0, 1.0, 10.0, eye, center, up)
        if self.context is NULL: 
            self.status = cCAPS.CAPS_NULLOBJ
            raise CAPSError(self.status, msg="while creating wvContext")
        
        self.context.keepItems = 1
        
        # May need to remove this/ change in the future 
        clearGlobal()
        
    
    @property
    def __context(self):
        return <object> self.context
    
    def __dealloc__(self):
        
        self.clearItems()
        
        if self.items:
            self.items = []
            
        clearGlobal()

        self.context = NULL
        
        self.controller = None
        
        self.graphicPrim = []
        self.graphicPrimName = []
        
        
    def __del__(self):
        self.clearItems()
        
        if self.items:
            self.items = []
        
        self.context = NULL
        
        self.controller = None
        
        self.graphicPrim = []
        self.graphicPrimName = []
    
    # Check to make sure all the primitives have the same number colors 
    def checkNumColor(self):
        print "Checking to make sure all the primitives have the same number of colors..."
        
        numColor = None
        for i in self.graphicPrim:
    
            if not numColor:
                numColor = len(i.colors)
            else:
                
                if numColor != len(i.colors):
                    
                    print "The total number of colors do NOT match for each primitive"
                    print "Primitive", i.name , "has", len(i.colors), "colors, while", numColor, \
                           "colors were found for another primitive!"
                    
                    self.status = cCAPS.CAPS_MISMATCH
                    raise CAPSError(self.status, msg="while checking the number of colors for each primitive")
        
        
        return numColor
    
    # Check and unify the limits for each color in the primitive 
    def checkUnifyLimits(self):
        
        maxVal = [] 
        minVal = [] 
        
        numColor = self.checkNumColor()
        print "Checking for a common max/min color limit all primitives..."
        
        for j in range(numColor):
            
            maxVal.append(-1E6)
            minVal.append(1E6)
     
            # Get the max value for all the current colors 
            for i in self.graphicPrim:
    
                item = i.colors[j]
                
                if item.minColor < minVal[j]:
                    minVal[j] = item.minColor
                    
                if item.maxColor > maxVal[j]:
                    maxVal[j] = item.maxColor
        
        for j in range(numColor):
            for i in self.graphicPrim:
                
                item = i.colors[j]
                
                item.updateColor(minVal[j], maxVal[j], item.numContour, item.reverseMap, item.colorMap)
    
    # Unify the bounding box for each primitive
    def unifyBoundingBox(self):
        
        print "Checking for a common bounding box for all primitives..."
    
        box = [1E6, -1E6, 1E6, -1E6, 1E6, -1E6]
        for i in self.graphicPrim:
            temp = i.vertex.boundingBox
            
            #print "Bounding Box = ", temp
            
            if temp[0] < box[0]:
                box[0] = temp[0]
            
            if temp[1] > box[1]:
                box[1] = temp[1]
            
            if temp[2] < box[2]:
                box[2] = temp[2]
            
            if temp[3] > box[3]:
                box[3] = temp[3]
                
            if temp[4] < box[4]:
                box[4] = temp[4]
            
            if temp[5] > box[5]:
                box[5] = temp[5]
        
        #print box
        
        focus = [0.5*(box[0]+box[1]), 0.5*(box[2]+box[3]), 0.5*(box[4]+box[5])]
        
        size = [box[1]-box[0], box[3]-box[2], box[5]-box[4]]
        focus.append(max(size))
        
        for i in self.graphicPrim:
            item = i.vertex
            item.setBoundingBox(box)
            item.adjustVertex(focus)
            
            i.updateVertex()
            
    ## Start the server. The port number specified when initializing the viewer object will be 
    # used (see \ref capsViewer.__init__ ). 
    # Note that the server will continue to run as long as the browser is still connected, once the connection is broken 
    # the capViewer object should be deleted!
    # See \ref webviewer.py for an example use case.
    #
    # \param enableCheck Enable checks to ensure all primitives have the same number colors and that the limits
    # for each color are initially the same. 
    def startServer(self, enableCheck = True):
        
        if enableCheck:
            # Check limits for primitives
            self.checkUnifyLimits()
            
        # Check bouding box and adjust focus 
        self.unifyBoundingBox()
        
        # Start server   
        self.status = cWV.wv_startServer(<int> self.portNumber, NULL, NULL, NULL, 0, self.context)
        if self.status < 0 :
            raise CAPSError(self.status-1000, msg="while starting server, code error "+str(self.status))
        
        self.serverInstance = self.status
            
        initial = True
        while cWV.wv_statusServer(0):

            time.sleep(0.5)
                
            if initial:
                # Open browser
                if self.controller:
                    self.controller.open(self.viewerHTML)
                
                initial = False
        
        #cWV.wv_removeAll(self.context)

        time.sleep(0.5)
        
        cWV.wv_cleanupServers()
    
    # Adjust the focus for all items.
    # \param unifyFocus Unify the focus for all items provided 
    # \param focus =  If a focal 
    #def adjustFocus(self, unifyFocus = True, focus=None):
    #    
    #    newFocus = [0, 0, 0, 0] 
    #    if unifyFocus:
# 
#             for i in self.items:
#                 
#                 if focus:
#                     newFocus = focus
#                 else:
#                     newFocus[0] = max(newFocus[0], i.focus[0])
#                     newFocus[1] = max(newFocus[1], i.focus[1])
#                     newFocus[2] = max(newFocus[2], i.focus[2])
#                     newFocus[3] = max(newFocus[3], i.focus[3])
#                
#                 i.setFocus(newFocus)
#                 i.adjustVertex()
#         else:
#             for i in self.items:
#                 
#                 if focus:
#                     i.setFocus(focus)
#              
#                 i.adjustVertex()
                
    
    def _determineBoundingBox(self, xyz):
        xyzMin = [ 99.0,  99.0,  99.0]
        xyzMax = [-99.0, -99.0, -99.0]
        if not isinstance(xyz, list):
            print "xyz should be a list"
            raise TypeError 
        
        if not isinstance(xyz[0], list):
            xyz = [xyz[:]]
            
        
        for i in range(len(xyz)):
            temp = xyz[i]
            if temp[0] < xyzMin[0]:
                xyzMin[0] = temp[0]
            
            if temp[0] > xyzMax[0]:
                xyzMax[0] = temp[0]
            
            if temp[1] < xyzMin[1]:
                xyzMin[1] = temp[1]
                
            if temp[1] > xyzMax[1]:
                xyzMax[1] = temp[1]
                
            if temp[2] < xyzMin[2]:
                xyzMin[2] = temp[2]
            
            if temp[2] > xyzMax[2]:
                xyzMax[2] = temp[2]
                    
        return [xyzMin[0], xyzMax[0], xyzMin[1], xyzMax[1], xyzMin[2], xyzMax[2]]
        
    ## Add a vertex set.
    # See \ref webviewer.py for an example use case.
    # \param xyz A list of lists of x,y, z values (e.g. [ [x2, y2, z2], [x2, y2, z2],[x3, y3, z3], etc. ] )
    # \param name Name of vertex set (default - None).
    # 
    # \return Optionally returns a reference to the "wvData item (object)" in which the data is stored. 
    def addVertex(self, xyz, name=None):

        if not isinstance(xyz, list):
            print "xyz should be a list"
            raise TypeError 
        
        if not isinstance(xyz[0], list):
            xyz = [xyz[:]]
        
        if name:
            self.items.append(wvData().createVertex(xyz, name = _byteify(str(name))))
        else:
            self.items.append(wvData().createVertex(xyz, name = NULL))
        self.currentItems.append(self.items[-1])
        
        self.items[-1].setBoundingBox(self._determineBoundingBox(xyz))
        
        return self.items[-1]
    
    ## Add a element connectivity set. 
    # See \ref webviewer.py for an example use case.
    # \param connectivity A list of lists of connectivity information 
    # (e.g. [ [node1, node2, node3], [node2, node3, node7, node8], etc. ]). Elements that >3 edges will internally
    # be decomposed into triangles. Note that whether or not the connectivity information is one biased was specified 
    # during capsViewer initialization (see \ref capsViewer.__init__ ), no further checks are currently made.
    # \param name Name of connectivity set (default - None). 
    #
    # \return Optionally returns a reference to the "wvData item (object)" in which the data is stored. 
    def addIndex(self, connectivity, name=None):
        
        if not isinstance(connectivity, list):
            print "connectivity should be a list"
            raise TypeError 
        
        if not isinstance(connectivity[0], list):
            connectivity = [connectivity[:]]
            
        if name:
            self.items.append(wvData().createIndex(connectivity, name = _byteify(str(name))))
        else:
            self.items.append(wvData().createIndex(connectivity, name = NULL))
            
        self.currentItems.append(self.items[-1])
        
        return self.items[-1]
    
    ## Add a element connectivity set with the intention of creating graphical lines on triangular primitives.
    # See \ref webviewer.py for an example use case.
    # \param connectivity A list of lists of connectivity information 
    # (e.g. [ [node1, node2, node3], [node2, node3, node7, node8], etc. ]). 
    # Note that whether or not the connectivity information is one biased was specified 
    # during capsViewer initialization (see \ref capsViewer.__init__ ), no further checks are currently made.
    # \param name Name of line set (default - None). 
    #
    # \return Optionally returns a reference to the "wvData item (object)" in which the data is stored. 
    def addLineIndex(self, connectivity, name = None):
        
        if not isinstance(connectivity, list):
            print "connectivity should be a list"
            raise TypeError 
        
        if not isinstance(connectivity[0], list):
            connectivity = [connectivity[:]]
            
        if name:
            self.items.append(wvData().createLineIndex(connectivity, name = _byteify(str(name))))
        else:
            self.items.append(wvData().createLineIndex(connectivity, name=NULL))
        
        self.currentItems.append(self.items[-1])
        
        return self.items[-1]
    
    ## Add color/scalar data set. 
    # See \ref webviewer.py for an example use case. 
    # \param colorData A list of color/scalar data to be applied at each node  
    # (e.g. [ node1_Color, node2_Color, node3_Color, etc.]). Cell-centered coloring isn't currently supported.
    #
    # \param minColor Minimum color value to use for scaling (default - None). If None, the minimum value is determined
    # automatically from the colorData provided. When plotting multiple color/data sets a minimum value should be provided
    # to ensure that all data is scaled the same. 
    #
    # \param maxColor Maximum color value to use for scaling (default - None). If None, the maximum value is determined
    # automatically from the colorData provided. When plotting multiple color/data sets a maximum value should be provided
    # to ensure that all data is scaled the same. 
    #
    # \param numContour Number of contour levels to use in the color map (default - 0 for a continuous color map).
    #
    # \param reverseMap Reverse or invert the color map (default - False).
    #
    # \param colorMap  Name of color map to use of visualization. 
    # Options: "blues" (default), "reds", "greys" (or "grays"), "blue_red" (or "bwr"), "jet", "rainbow", "hot", "cool".  
    #
    # \param name Name of color set (default - None). 
    #
    # \return Optionally returns a reference to the "wvData item (object)" in which the data is stored. 
    def addColor(self, colorData, minColor=None, maxColor=None, numContour = 0, reverseMap = False, colorMap ="blues", name = None):
        
        if not isinstance(colorData, list):
            colorData = [colorData]
        
        if not minColor:
            minColor = min(colorData)
        if not maxColor:
            maxColor = max(colorData)
        
        if name:
            self.items.append(wvData().createColor(colorData, float(minColor), float(maxColor), 
                                                   int(numContour), int(reverseMap),
                                                   colorMap = _byteify(colorMap.lower()), name = _byteify(str(name))))
        else:
            self.items.append(wvData().createColor(colorData, float(minColor), float(maxColor), 
                                                   int(numContour), int(reverseMap),
                                                   colorMap = _byteify(colorMap.lower()), name = NULL))
        
        self.currentItems.append(self.items[-1])
        
        return self.items[-1]    
    
    ## Add color/scalar data set for the graphical lines on triangular primitives. (\ref addLineIndex ). 
    # See \ref webviewer.py for an example use case. 
    # \param colorData A list of color/scalar data to be applied at each node
    # (e.g. [ node1_Color, node2_Color, node3_Color, etc.]).
    #
    # \param minColor Minimum color value to use for scaling (default - None). If None, the minimum value is determined
    # automatically from the colorData provided. When plotting multiple color/data sets a minimum value should be provided
    # to ensure that all data is scaled the same. 
    #
    # \param maxColor Maximum color value to use for scaling (default - None). If None, the maximum value is determined
    # automatically from the colorData provided. When plotting multiple color/data sets a maximum value should be provided
    # to ensure that all data is scaled the same. 
    #
    # \param numContour Number of contour levels to use in the color map (default - 0 for a continuous color map).
    #
    # \param reverseMap Reverse or invert the color map (default - False).
    # 
    # \param colorMap  Name of color map to use of visualization. 
    # Options: "blues" (default), "reds", "greys" (or "grays"), "blue_red" (or "bwr"), "jet", "rainbow", "hot", "cool".  
    #
    # \param name Name of color set (default - None). 
    #
    # \return Optionally returns a reference to the "wvData item (object)" in which the data is stored. 
    def addLineColor(self, colorData, minColor=None, maxColor=None, numContour = 0, reverseMap = False, colorMap ="blues", name=None):
        
        if not isinstance(colorData, list):
            colorData = [colorData]
            
        if not minColor:
            minColor = min(colorData)
        if not maxColor:
            maxColor = max(colorData)
        
        if name:
            self.items.append(wvData().createLineColor(colorData, float(minColor), float(maxColor), 
                                                       int(numContour), int(reverseMap),
                                                       colorMap = _byteify(colorMap.lower()), name = _byteify(str(name))))
        
        else:
            self.items.append(wvData().createLineColor(colorData, float(minColor), float(maxColor), 
                                                       int(numContour), int(reverseMap),
                                                       colorMap = _byteify(colorMap.lower()), name=NULL))
            
        self.currentItems.append(self.items[-1])
        
        return self.items[-1]    
    
    ## Create a generic graphic primitive.
    # \param type Type of primitive to create. Options: "point", "line", or "triangle" (case insensitive)
    # \param items List of "wvData items (objects)" used to create the triangle (default - None). 
    # If none are provided all current items added will be used! 
    #
    # \param name Name of graphic primitive (default - GPrim_#, where # is 1 + number of primitives previously loaded). 
    # Important: If specified, the name must be unique.
    #
    # \return Optionally returns a reference to the graphicPrimitive object. 
    def createPrimitive(self, type, items=None, name=None ):
        if name is None:
            name = "GPrim_" + str(len(self.graphicPrim)+1)
             
        if items == None:
            items = self.currentItems #self.items
        
        gPrim = graphicPrimitive(self)
        
        if not isinstance(name, bytes) or isinstance(name, str):
            name = str(name)
    
        if type.lower() == "point":
            gPrim.createPoint(items, _byteify(name))
        elif type.lower() == "line":
            gPrim.createLine(items, _byteify(name))
        elif type.lower() == "triangle":
            gPrim.createTriangle(items, _byteify(name))
        else:
            raise TypeError
        self.graphicPrim.append(gPrim)
        
        global globalInstance
        globalInstance.numPrim = len(self.graphicPrim)
        
        globalInstance.gprim = <void **> realloc(globalInstance.gprim, globalInstance.numPrim*sizeof(void *))
        globalInstance.gprim[globalInstance.numPrim-1] = <void *> gPrim
        
        return gPrim
        
    ## Create a point graphic primitive.
    # \param items List of "wvData items (objects)" used to create the triangle (default - None). 
    # If none are provided all current items added will be used! 
    #
    # \param name Name of graphic primitive (default - GPrim_#, where # is 1 + number of primitives previously loaded). 
    # Important: If specified, the name must be unique.
    #
    # \return Optionally returns a reference to the graphicPrimitive object. 
    def createPoint(self, items=None, name=None):
    # \return Index of graphic primitive
        
        return self.createPrimitive("point", items=items, name=name)
        
    ## Create a line graphic primitive.
    # \param items List of "wvData items (objects)" used to create the triangle (default - None). 
    # If none are provided all current items added will be used! 
    #
    # \param name Name of graphic primitive (default - GPrim_#, where # is 1 + number of primitives previously loaded). 
    # Important: If specified, the name must be unique.
    #
    # \return Optionally returns a reference to the graphicPrimitive object. 
    def createLine(self, items=None, name=None, turnOn=True):
    # \return Index of graphic primitive
        
        return self.createPrimitive("line", items=items, name=name)
        
    ## Create a triangular graphic primitive.
    # See \ref webviewer.py for an example use case.
    # \param items List of "wvData items (objects)" used to create the triangle (default - None). 
    # If none are provided all current items added will be used! 
    #
    # \param name Name of graphic primitive (default - GPrim_#, where # is 1 + number of primitives previously loaded). 
    # Important: If specified, the name must be unique.
    #
    # \return Optionally returns a reference to the graphicPrimitive object. 
    def createTriangle(self, items=None, name=None, turnOn=True):
   
        
        return self.createPrimitive("triangle", items=items, name=name)
        
    ## Clear the "current" list of "wvData items (objects)".
    def clearItems(self):
        
        if self.currentItems:
            self.currentItems = []

        #if self.items:
        #   self.items = []
         
    ## Add a _capsDataSet (see \ref _capsDataSet) object(s). A graphic primitive will be created automatically for
    # each data set. Note, however, if multiple data sets share the same vertex set (see \ref _capsVertexSet) 
    # the data in the repeated vertex sets will be appended into a single primitive.
    # 
    # \param dataSet A single or list of instances of _caspDataSet objects. 
    #
    def addDataSet(self, dataSet):
        
        if not isinstance(dataSet, list):
            dataSet = [dataSet]
        
        # Clear out list of current items
        self.clearItems()
        
        #xyzMin = [ 99.0,  99.0,  99.0]
        #xyzMax = [-99.0, -99.0, -99.0]
        
        boundVertexDict = {}
            
        colorRange = {}
        
        # Setup colorRange dictionary
        for i in dataSet:
            
            # Check to make sure data sets are actually data sets 
            if not isinstance(i, _capsDataSet): 
                print "Value is not a _capsDataSet"
                raise TypeError
            
            for k in range(i.dataRank):
                colorRange["Name="+i.dataSetName+";Rank="+str(k+1)] = {"Min" : 1E6, "Max" : -1E6}
            
        # Get max and min XYZ and max and min color
        for i in dataSet:
             
            if not isinstance(i, _capsDataSet): # This was already checked, supposedly 
                print "Value is not a _capsDataSet"
                raise TypeError
       
            #xyz = i.getDataXYZ()
            
            dataVal = i.getData()
            dataRank = i.dataRank
            
            #for j in range(len(xyz)):
            #    temp = xyz[j]
            #    if temp[0] < xyzMin[0]:
            #        xyzMin[0] = temp[0]
            #    
            #    if temp[0] > xyzMax[0]:
            #        xyzMax[0] = temp[0]
            #    
            #    if temp[1] < xyzMin[1]:
            #        xyzMin[1] = temp[1]
            #        
            #    if temp[1] > xyzMax[1]:
            #        xyzMax[1] = temp[1]
            #        
            #    if temp[2] < xyzMin[2]:
            #        xyzMin[2] = temp[2]
            #    
            #    if temp[2] > xyzMax[2]:
            #       xyzMax[2] = temp[2]
        
            if dataRank == 1:
                k = 0
                for val in dataVal:
                    if val < colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"]:
                        
                        colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"] = val 
                    
                    if val > colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"]:
                    
                        colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"] = val
                    
            else:
                
                for j in dataVal:
                    for k, val in enumerate(j):
                    
                        if val < colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"]:
                        
                            colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"] = val 
                    
                        if val > colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"]:
                    
                            colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"] = val
        
        #print xmin, ymin, zmin, xmax, ymax, zmax
        
        #focus = [0.5*(xyzMin[0]+xyzMax[0]), 0.5*(xyzMin[1]+xyzMax[1]), 0.5*(xyzMin[2]+xyzMax[2])]
        #focus.append(max(focus))
        
        # Add triangles 
        for i in dataSet:
            
            if not isinstance(i, _capsDataSet): # This was already checked, supposedly 
                print "Value is not a _capsDataSet"
                raise TypeError
       
            self.clearItems()
            
            boundVertex =  "Bound=" + i.capsBound.boundName + ";Vertex=" + i.capsVertexSet.vertexSetName
            # Create the primitive, but only if it is a unique vertex set  
            if boundVertex not in boundVertexDict.keys():
            
                boundVertexDict[boundVertex] = None
                     
                self.addVertex(i.getDataXYZ())
            
                #item.adjustVertex(focus)
            
                self.addIndex(i.getDataConnect()[0])
                self.addLineIndex(i.getDataConnect()[0])
                self.addLineColor(1, colorMap = "grey")
            
                if i.dataRank == 1:
                    k = 0
                    self.addColor(i.getData(), 
                                  minColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"], 
                                  maxColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"], 
                                  name = _byteify(str(i.dataSetName)))
    
                else:
                    
                    dataVal = i.getData()
                
                    for k in range(i.dataRank):
                        temp = []
                        
                        # Lets create a color map for all ranks, createPrimative will only show the first one
                        for val in dataVal:  
                            temp.append(val[k])
                            
                        name = _byteify(i.dataSetName+";Rank="+str(k+1))
                        #print "Initial", name 
                        self.addColor(temp, 
                                      minColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"], 
                                      maxColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"], 
                                      name = name)
                
                name = _byteify(str(i.capsBound.boundName))
                #print "Name = " , name
                boundVertexDict[boundVertex] = self.createTriangle(name=name)
            
            else: # Lets just create the color for the data set and append it to the primitive that already exists
                
                if i.dataRank == 1:
                    k = 0
                    item = self.addColor(i.getData(), 
                                         minColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"], 
                                         maxColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"], 
                                         name = _byteify(str(i.dataSetName)))
                    
                    boundVertexDict[boundVertex].appendColor(item)
    
                else:
                    
                    dataVal = i.getData()
                
                    for k in range(i.dataRank):
                        temp = []
                        
                        # Lets create a color map for all ranks, createPrimative will only show the first one
                        for val in dataVal:  
                            temp.append(val[k])
                            
                        name = _byteify(i.dataSetName+";Rank="+str(k+1))
                        #print "Append", name 
                        item = self.addColor(temp, 
                                             minColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Min"], 
                                             maxColor=colorRange["Name="+i.dataSetName+";Rank="+str(k+1)]["Max"], 
                                             name = name)
                            
                        boundVertexDict[boundVertex].appendColor(item)
                        
    ## Alias to \ref addDataBound
    def addBound(self, bound, dataSetType="both"):
        self.addDataBound( bound, dataSetType="both")

    ## Add a _capsBound (see \ref _capsBound) object(s). 
    #
    # \param bound A single or list of instances of _capsBound objects. 
    # \param dataSetType Specifies which type of data sets in the bound should be added, source or destination.
    # Options: "source", "destination, or "both" (default) - values are case insensitive. 
    def addDataBound(self, bound, dataSetType="both"):  
        
        # Covert to list if not already one
        if not isinstance(bound, list):
            bound = [bound]
            
        dataSetType = dataSetType.lower()

        # Check data set type
        if dataSetType not in ["both", "source", "destination"]:
            
            print "Invalid option for dataSetType"
            raise ValueError
        
        # Initialize date set list
        dataSetList = []

        for i in bound:
            
            # Check to make sure the bound is actually a bound 
            if not isinstance(i, _capsBound):
                print "Value is not a _capsBound"
                raise TypeError
            
            if dataSetType == "both" or dataSetType == "source":
                
                dataSetList += i.dataSetSrc.values()
                
            if dataSetType == "both" or dataSetType == "destination":
                
                dataSetList += i.dataSetDest.values()
                
                    
        # Pass the list of data sets to addDataSet - it is supposed to do all the sorting 
        self.addDataSet(dataSetList)
    
    ## Add a Tecplot file.
    # 
    # \param filename Name of Tecplot file to load. 
    def addTecplot(self, filename):
        
        def determineXYZKey(variables, xyzName):
            
            for i in variables:
                
                if xyzName == i.lower():
                    return i
                elif xyzName + " " in i.lower():
                    return i 
                else:
                    pass
            
            print "Unable to determine which variable is x, y, or z "
            raise ValueError
            
        tp = capsTecplot(filename)
        tp.loadZone()
        
        # Check for types 
        for i in tp.zone.values():
            
            if ( i.zoneType != "FELINESEG"  and 
                 i.zoneType != "FETRIANGLE" and 
                 i.zoneType != "FEQUADRILATERAL"):
                
                print "Unsupported zone type - ", i.zoneType
                raise TypeError
        
        # Clear out list of current items
        self.clearItems()
        
        colorRange = {}
        
        # Setup colorRange dictionary
        for k in tp.variable:
            colorRange[k] = {"Min" : 1E6, "Max" : -1E6}
        
        xyzName = [determineXYZKey(tp.variable, "x"), 
                   determineXYZKey(tp.variable, "y"),
                   determineXYZKey(tp.variable, "z")]
        
        # Get max and min color
        for i in tp.zone.values():
            
            for j in tp.variable:
                
                # Skip x, y, z variables 
                if j in xyzName:
                    continue
                
                for val in i.data[j]:
                    if val < colorRange[j]["Min"]:
                        
                        colorRange[j]["Min"] = val 
                    
                    if val > colorRange[j]["Max"]:
                    
                        colorRange[j]["Max"] = val
            
        # Add triangles 
        for i in tp.zone.values():

            self.clearItems()

            # Copy over xyz to put in [[x1, y1, z1], [x2, y2, z2], etc... ] format
            xyz = []
            for j in range(i.numNode):
                xyz.append([i.data[xyzName[0]][j], 
                            i.data[xyzName[1]][j], 
                            i.data[xyzName[2]][j]])
                
            self.addVertex(xyz)
            
            #item.adjustVertex(focus)
            
            self.addIndex(i.connectivity)
            
            self.addLineIndex(i.connectivity)
            
            self.addLineColor(1, colorMap = "grey")
            
            for j in tp.variable:
                
                if j in xyzName:
                    continue
                
                self.addColor(i.data[j], 
                              minColor=colorRange[j]["Min"], 
                              maxColor=colorRange[j]["Max"], 
                              name = _byteify(str(j)))
            
            name = _byteify(str(i.zoneName.rstrip().lstrip().replace("'", "").replace('"', "").replace(" ", "")))
            #print "Name = " , name
            self.createTriangle(name=name)
    
    ## Add a unstructured mesh file. Note that in most cases 
    # the mesh's name (see \ref capsUnstructMesh.$name) will be used for the name of the primitive.
    # 
    # \param meshFile Name of unstructured mesh file to load or an instance of a 
    # capsUnstructMesh(see \ref capsUnstructMesh) object.
    #  
    def addUnstructMesh(self, meshFile):
        
        if isinstance(meshFile, str):
            mesh = capsUnstructMesh(meshFile)
        elif isinstance(meshFile, capsUnstructMesh):
            mesh = meshFile 
        
        if mesh.numNode == 0 or mesh.numElement == 0:    
            mesh.loadFile()
        
        if mesh.oneBias != self.oneBias:
            print "Index biasing between mesh and server mismatch!"
            raise ValueError
        
        # Clear out list of current items
        self.clearItems()
        
        colorRange = {}
        
        # Setup colorRange dictionary
        for i in mesh.data.keys():
            colorRange[i] = {"Min" : 1E6, "Max" : -1E6}
        
        # Get max and min color
        for i in mesh.data.keys():
            
            for val in mesh.data[i]:
                if val < colorRange[i]["Min"]:
                        
                    colorRange[i]["Min"] = val 
                    
                if val > colorRange[i]["Max"]:
                    
                    colorRange[i]["Max"] = val
            
        # Origami mesh
        if mesh.meshType == "origami":
            
            # Add lines - create a new primitive for each line so we can individually select the color
            for index, i in enumerate(mesh.connectivity):
                
                self.clearItems()
                
                xyz = [mesh.xyz[i[0]-1], mesh.xyz[i[1]-1]]
                 
                self.addVertex(xyz)
                
                #item.adjustVertex(focus)
                
                self.addIndex([1, 2])
                
                for j in mesh.data.keys():
                    self.addLineColor(mesh.data[j][index], 
                                      minColor=colorRange[j]["Min"], 
                                      maxColor=colorRange[j]["Max"], 
                                      name = _byteify(str(j)),
                                      colorMap = "rainbow")
                    
                name = _byteify(str(mesh.name + str(index)))
                #print "Name = " , name
                self.createLine(name=name)
        
        elif mesh.meshType == "aflr3": # We can only handle surface meshes
            
            # Count how many triangle and quadrilaterals there are 
            
            count = len(mesh.data["BCID"])
            
            #print count, mesh.numElement
            
            if count != mesh.numElement:
                print "capsViewer can only visualize surface data - volume elements will be ignored!"
                
                # Determine what the last node in the connectivty should be 
                maxIndex = 0
                for i in range(count):
                    
                    temp = max(mesh.connectivity[i])
                    if  temp > maxIndex:
                        maxIndex = temp
                        
                #print "Maxindex", maxIndex
                
                # Assume first n-nodes just belong to the surface
                self.addVertex(mesh.xyz[0:maxIndex])
                
                # First e- elements are triangles and quadrilaterals
                self.addIndex(mesh.connectivity[0:count])
                self.addLineIndex(mesh.connectivity[0:count])
            
            else:
                
                self.addVertex(mesh.xyz)
             
                self.addIndex(mesh.connectivity)
                self.addLineIndex(mesh.connectivity)
            
            self.addLineColor(1, colorMap = "grey")
            self.addColor(1, colorMap = "blue") # Temporary - need to push boundary ids to nodes
            #for i in mesh.data.keys():
            #    self.addColor(mesh.data[i], 
            #                  minColor=colorRange[i]["Min"], 
            #                  maxColor=colorRange[i]["Max"], 
            #                  name = _byteify(str(i)))
                 
            name = _byteify(str(mesh.name))
            #print "Name = " , name
            self.createTriangle(name=name)
        
        elif mesh.meshType == "stl":
            
            self.addVertex(mesh.xyz)
            self.addIndex(mesh.connectivity)
            self.addLineIndex(mesh.connectivity)
            self.addLineColor(1, colorMap = "grey")
            self.addColor(1, colorMap = "blue")

            # Would need to convert element data to nodes 
#             for i in mesh.data.keys():
#                 self.addColor(mesh.data[i], 
#                               minColor=colorRange[i]["Min"], 
#                               maxColor=colorRange[i]["Max"], 
#                               name = _byteify(str(i)))
#                 
            name = _byteify(str(mesh.name))
            #print "Name = " , name
            self.createTriangle(name=name)

        # Default mesh 
        else:
            
            print "Mesh type", mesh.meshType, "isn't reconzied!"
            raise TypeError

#             self.addVertex(mesh.xyz)
#             self.addIndex(mesh.connectivity)
#             self.addLineIndex(mesh.connectivity)
#             self.addLineColor(1, colorMap = "grey")
#                 
#             for i in mesh.data.keys():
#                 self.addColor(mesh.data[i], 
#                               minColor=colorRange[i]["Min"], 
#                               maxColor=colorRange[i]["Max"], 
#                               name = _byteify(str(i)))
#                 
#             name = _byteify(str(mesh.name))
#             #print "Name = " , name
#             self.createTriangle(name=name)