/*
 ************************************************************************
 *                                                                      *
 * Fitter.h -- header file for Fitter                                   *
 *                                                                      *
 *              Written by John Dannenhoffer @ Syracuse University      *
 *                                                                      *
 ************************************************************************
 */

/*
 * Copyright (C) 2011/2018  John F. Dannenhoffer, III (Syracuse University)
 *
 * This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *     MA  02110-1301  USA
 */

#ifndef _FITTER_H_
#define _FITTER_H_

int    fit1dCloud(int    m,             /* (in)  number of points in cloud */
                  double XYZcloud[],    /* (in)  array  of points in cloud */
                  int    n,             /* (in)  number of control points */
                  double cp[],          /* (in)  array  of control points (first and last are set) */
                                        /* (out) array  of control points (all set) */
                  double Tcloud[],      /* (out) T-parameters of points in cloud */
                  double *normf,        /* (out) RMS of distances between cloud and fit */
                  double *dotmin,       /* (out) minimum normalized dot product of control polygon */
                  int    *nmin,         /* (out) minimum number of cloud points in any interval */
                  int    *numiter,      /* (out) number of iterations executed */
                  FILE   *fp);          /* (in)  file for progress outputs (or NULL) */

int    fit2dCloud(int    m,             /* (in)  number of points in cloud */
                  double XYZcloud[],    /* (in)  array  of points in cloud */
                  int    n,             /* (in)  number of control points in each direction */
                  double cp[],          /* (in)  array  of control points (boundaries set) */
                                        /* (out) array  of control points (all set) */
                  double UVcloud[],     /* (out) UV-parameters of points in cloud */
                  double *normf,        /* (out) RMS of distances between cloud and fit */
                  int    *nmin,         /* (out) minimum number of cloud points in any interval */
                  int    *numiter,      /* (out) number of iterations executed */
                  FILE   *fp);          /* (in)  file for progress outputs (or NULL) */

#ifdef GRAFIC
int    plotCurve(int    m,              /* (in)  number of points in cloud */
                 double XYZcloud[],     /* (in)  array  of points in cloud */
       /*@null@*/double Tcloud[],       /* (in)  T-parameters of points in cloud (or NULL) */
                 int    n,              /* (in)  number of control points */
                 double cp[],           /* (in)  array  of control points */
                 double normf,          /* (in)  RMS of distances between cloud and fit */
                 double dotmin,         /* (in)  minimum noralized dot product of control polygon */
                 int    nmin);          /* (in)  minimum number of cloud points in any interval */

int    plotSurface(int    m,            /* (in)  number of points in cloud */
                   double XYZcloud[],   /* (in)  array  of points in cloud */
         /*@null@*/double UVcloud[],    /* (in)  UV-parameters of points in cloud (or NULL) */
                   int    n,            /* (in)  number of control points in each direction */
                   double cp[],         /* (in)  array  of control points */
                   double normf,        /* (in)  RMS of distances between cloud and fit */
                   int    nmin);        /* (out) minimum number of cloud points in any interval */
#endif

#endif   /* _FITTER_H_ */
